package gov.va.nvap.common.sql;

/**
 * Contains common functions for SQL manipulation.
 * 
 * @since 05/13/2015
 * @author                  
 */
public class SqlUtil {
    /**
     * Generates part of the WHERE clause so that test patients are excluded from the query.
     * 
     * @param useHibernate Whether to use Hibernate syntax
     * @param tableName Table name or alias to use
     * @param ssnColumnName SSN column name or alias to use
     * @return SQL clause to be inserted into the WHERE clause
     */
    public static String generateExcludeTestPatientsClause(boolean useHibernate, String tableName, String ssnColumnName) {
        String col = tableName + "." + ssnColumnName;
        String substr = useHibernate ? "substring" : "substr";
        return "(" +
                 col + " is null or " +                          // Ignore null entries
                "(" +
                  substr + "(" + col + ", 1, 3) != '000' and " + // Area number 000 denotes a test patient
                  substr + "(" + col + ", 1, 3) != '666' and " + // Area number 666 denotes a test patient
                  substr + "(" + col + ", 1, 1) != '9'   and " + // Area number in 900-999 range denotes a test patient
                  substr + "(" + col + ", 4, 2) != '00'  and " + // Group number 00 denotes a test patient
                  substr + "(" + col + ", 6, 4) != '0000'" +     // Serial number 0000 denotes a test patient
                ")" +
               ")";
    }
}
