DECLARE tableCount NUMBER;
        seqCount NUMBER;

BEGIN

  SELECT COUNT('x') INTO tableCount
  FROM USER_ALL_TABLES
  WHERE TABLE_NAME = 'PATIENT_PROFILE';

  IF (tableCount = 0) THEN
      -- the table does not exist so we need to create it
      EXECUTE IMMEDIATE 'CREATE TABLE PATIENT_PROFILE
      (
        IEN VARCHAR2(255 CHAR) NOT NULL,
        LAST_NAME VARCHAR2(100 CHAR) NOT NULL,
        FIRST_NAME VARCHAR2(100 CHAR) NOT NULL,
        MIDDLE_INITIAL CHAR(1) NULL,
        DOB DATE NOT NULL,
        GENDER VARCHAR2(10 CHAR) NOT NULL,
        SSN VARCHAR2(20 CHAR) NULL,
        FACILITY_ID NUMBER NOT NULL, 
        PRIMARY KEY (IEN),
        CONSTRAINT FK_PATIENT_PROFILE_FACILITY
          FOREIGN KEY (FACILITY_ID)
          REFERENCES FACILITY(FACILITY_ID)
      )
      TABLESPACE "USERS"';  
      
      -- clear out all the garbage lookup and other data we don't want around
      DELETE FROM PATIENT_DOCUMENT;

      DELETE FROM CONSENT_AUDIT;

      DELETE FROM PATIENT_EXCLUDED_ORG;
      
      DELETE FROM PATIENT_CONSENT_DIR;
      
      DELETE FROM CONSENT_TYPE;
      
      DELETE FROM PURPOSE_OF_USE;
      
      DELETE FROM OPTOUT_REASON;
      
      -- and now we need to populate some of them
      
      -- do the insert
      INSERT INTO PURPOSE_OF_USE
      (POU_ID, POU_VALUE, URN)
      VALUES
      (100, 'TREATMENT', 'urn:oasis:names:tc:xspa:1.0:subject:purposeofuse');
      
      INSERT INTO PURPOSE_OF_USE
      (POU_ID, POU_VALUE, URN)
      VALUES
      (101, 'EMERGENCY', 'urn:oasis:names:tc:xspa:1.0:subject:purposeofuse');
      
      INSERT INTO PURPOSE_OF_USE
      (POU_ID, POU_VALUE, URN)
      VALUES
      (102, 'COVERAGE', 'urn:oasis:names:tc:xspa:1.0:subject:purposeofuse');
      
      -- reset the sequence
      BEGIN
        EXECUTE IMMEDIATE 'DROP SEQUENCE PURPOSE_OF_USE_SEQ';
      
        EXECUTE IMMEDIATE 'CREATE SEQUENCE PURPOSE_OF_USE_SEQ MINVALUE 1 MAXVALUE 9999999999999999999999999999 INCREMENT BY 1 START WITH 103 CACHE 20 NOORDER';
      END;
      -- do the insert
      INSERT INTO CONSENT_TYPE
      (CONSENT_TYPE_ID, NAME)
      VALUES
      (100, 'NwHIN Authorization');
      
      INSERT INTO CONSENT_TYPE
      (CONSENT_TYPE_ID, NAME)
      VALUES
      (101, 'NwHIN Revocation');
      
      INSERT INTO CONSENT_TYPE
      (CONSENT_TYPE_ID, NAME)
      VALUES
      (102, 'SSA Authorization');
      
      INSERT INTO CONSENT_TYPE
      (CONSENT_TYPE_ID, NAME)
      VALUES
      (103, 'SSA Revocation');
      
      INSERT INTO CONSENT_TYPE
      (CONSENT_TYPE_ID, NAME)
      VALUES
      (108, 'NwHIN Organization Restriction Authorization');
      
      INSERT INTO CONSENT_TYPE
      (CONSENT_TYPE_ID, NAME)
      VALUES
      (109, 'NwHIN Organization Restriction Revocation');
      
      -- reset the sequence
      BEGIN
        EXECUTE IMMEDIATE 'DROP SEQUENCE CONSENT_TYPE_SEQ';
        
        EXECUTE IMMEDIATE 'CREATE SEQUENCE CONSENT_TYPE_SEQ MINVALUE 1 MAXVALUE 9999999999999999999999999999 INCREMENT BY 1 START WITH 110 CACHE 20 NOORDER';
      END;
      
      -- do the insert
      INSERT INTO OPTOUT_REASON
      (OPTOUT_REASON_ID, TEXT)
      VALUES
      (100, 'Entered in Error');
      
      INSERT INTO OPTOUT_REASON
      (OPTOUT_REASON_ID, TEXT)
      VALUES
      (101, 'Patient Deceased');
      
      INSERT INTO OPTOUT_REASON
      (OPTOUT_REASON_ID, TEXT)
      VALUES
      (102, 'Revoked');
      
      INSERT INTO OPTOUT_REASON
      (OPTOUT_REASON_ID, TEXT)
      VALUES
      (20000001, 'Authorization Expired');
      
      INSERT INTO OPTOUT_REASON
      (OPTOUT_REASON_ID, TEXT)
      VALUES
      (20000002, 'New Authorization');
      
      -- reset the sequence
      BEGIN
        SELECT COUNT('x') INTO seqCount
        FROM user_sequences 
        WHERE sequence_name = 'CONSENT_AUDIT_SEQ';
        
        IF( seqCount > 0) THEN
          EXECUTE IMMEDIATE 'DROP SEQUENCE OPTOUT_REASON_SEQ';
        END IF;
        
        EXECUTE IMMEDIATE 'CREATE SEQUENCE OPTOUT_REASON_SEQ MINVALUE 1 MAXVALUE 9999999999999999999999999999 INCREMENT BY 1 START WITH 20000003 CACHE 20 NOORDER';
      END;      
      
      -- create a sequence for the CONSENT_AUDIT table
      BEGIN
        SELECT COUNT('x') INTO seqCount
        FROM user_sequences 
        WHERE sequence_name = 'CONSENT_AUDIT_SEQ';
        
        IF( seqCount > 0) THEN
          EXECUTE IMMEDIATE 'DROP SEQUENCE CONSENT_AUDIT_SEQ';
        END IF;
        
        EXECUTE IMMEDIATE 'CREATE SEQUENCE CONSENT_AUDIT_SEQ MINVALUE 1 MAXVALUE 9999999999999999999999999999 INCREMENT BY 1 START WITH 1 CACHE 20 NOORDER';
      END; 
  END IF;

END;
