/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package gov.va.nvap.server.auth;

import gov.va.nvap.service.auth.ServiceAudit;
import gov.va.nvap.service.auth.ServiceAuditDAO;
import gov.va.nvap.service.auth.ServiceAuthenticationException;
import gov.va.nvap.service.auth.ServicePermission;
import gov.va.nvap.service.auth.ServicePermissionDAO;
import java.io.IOException;
import java.security.cert.X509Certificate;
import java.util.Date;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.annotation.Resource;
import javax.interceptor.Interceptors;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.xml.ws.WebServiceContext;
import javax.xml.ws.handler.MessageContext;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.ApplicationContext;
import org.springframework.ejb.interceptor.SpringBeanAutowiringInterceptor;

/**
 *
 * @author 564685
 */
@Interceptors(SpringBeanAutowiringInterceptor.class)
public class ServiceAuthentication{
    public boolean checkAuth(WebServiceContext context,ApplicationContext applicationContext, String function) throws ServiceAuthenticationException{
        boolean authorized = false;
        HttpServletRequest hsr;
        
        try { 
            hsr = (HttpServletRequest)context.getMessageContext().get(MessageContext.SERVLET_REQUEST);
        } catch (IllegalStateException e) {
            return true; //Internal call
        }
        X509Certificate[] certs = (X509Certificate[])hsr.getAttribute("javax.servlet.request.X509Certificate");
        if(certs != null && certs.length > 0){ 
           
            ServicePermissionDAO spdao = applicationContext.getBean(ServicePermissionDAO.class);
            String dn = certs[0].getSubjectDN().getName();
            String address = hsr.getRemoteAddr();
            String servelet = hsr.getServletPath();
            String cn = dn.split(",")[0].substring(3).trim();
            ServicePermission sp = spdao.checkAuthorization(cn);
            if(sp != null && sp.getEnabled()){
                authorized = true;
            }else{
                sp = spdao.checkAuthorization("*wildcard*");
                if(sp != null && sp.getEnabled()){
                    Logger.getLogger(ServiceAuthentication.class.getName()).log(Level.INFO, "("+cn+ ") does not match any allowed Authorized Service but wildcard is enabled.");
                    authorized = true;
                }
            }

            ServiceAuditDAO sadao = applicationContext.getBean(ServiceAuditDAO.class);
            ServiceAudit sa = new ServiceAudit();
            sa.setEventDate(new Date());
            sa.setCN(cn);
            sa.setIp(address);
            sa.setWebService(servelet.substring(1));
            sa.setCall(function);
            sa.setAuthorized(authorized);
            try {
                sadao.create(sa);
            } catch (Exception ex) {
                Logger.getLogger(ServiceAuthentication.class.getName()).log(Level.SEVERE, "Unable to store service audit", ex);
            }
            if(!authorized){ 
                this.returnError(context, "Application is not authorized");
            }
        }
        else{
           this.returnError(context, "Certificate was not passed");
        }
        return authorized;
    }
    private void returnError(WebServiceContext context,String message) throws ServiceAuthenticationException{
        HttpServletResponse hsr = (HttpServletResponse)context.getMessageContext().get(MessageContext.SERVLET_RESPONSE);
        try {
            hsr.sendError(403, message);
        } catch (IOException ex) { 
                 Logger.getLogger(ServiceAuthentication.class.getName()).log(Level.SEVERE, "Unable to return 403 response");
        }
         throw new ServiceAuthenticationException(message);
    }
}
