package gov.va.nvap.server.endpoint.announce;

import gov.va.med.nhin.adapter.announce.BatchAnnounceType;
import gov.va.nvap.common.validation.NullChecker;
import gov.va.nvap.server.endpoint.adapter.announce.transformer.AnnouncementToAnnouncePatientRequest;
import gov.va.nvap.service.pdq.PatientDemographicsQuery;
import gov.va.nvap.svc.consentmgmt.stub.adapter.announce.data.Announcement;
import gov.va.nvap.svc.consentmgmt.stub.adapter.announce.data.AnnouncerInterface;
import gov.va.nvap.svc.consentmgmt.stub.adapter.announce.data.PatientAnnouncer;
import gov.va.nvap.svc.consentmgmt.stub.adapter.announce.jaxb.AnnouncePatientRequest;
import gov.va.nvap.svc.consentmgmt.stub.adapter.announce.jaxb.PatientType;
import gov.va.nvap.svc.consentmgmt.stub.adapter.announce.jaxb.UserType;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Date;
import java.util.List;

import javax.annotation.Resource;
import javax.ejb.ActivationConfigProperty;
import javax.ejb.EJB;
import javax.ejb.MessageDriven;
import javax.ejb.MessageDrivenContext;
import javax.ejb.TransactionAttribute;
import javax.ejb.TransactionAttributeType;
import javax.jms.Message;
import javax.jms.MessageListener;
import javax.jms.ObjectMessage;

/**
 * Message-Driven Bean implementation class for: BatchAnnouncer
 * 
 */
@MessageDriven(mappedName = "jms.vap.PatientAnnouncerBatchQueue", activationConfig = {
		@ActivationConfigProperty(propertyName = "acknowledgeMode", propertyValue = "Auto-acknowledge"),
		@ActivationConfigProperty(propertyName = "destinationType", propertyValue = "javax.jms.Queue") })
@TransactionAttribute(TransactionAttributeType.REQUIRES_NEW)
public class BatchAnnouncerMessageConsumer implements MessageListener {

	private static final int MAX_BATCH_SIZE = 1000;

	@EJB(name = "AnnouncerService", mappedName = "AnnouncerService")
	private AnnouncerInterface announcer;

	@EJB(name = "PatientAnnouncer", mappedName = "PatientAnnouncer")
	private PatientAnnouncer patientAnnouncer;

	private void announce(final String[] batchIds, final String userName,
			final String facility, final int maxBatchSize) throws Exception {
		try {
			for (final String batchId : batchIds) {
				Collection<Announcement> announcements = null;
				int firstResult = 0;
				do {
					announcements = this.announcer
							.getUnscheduledAnnouncementsByBatchId(batchId,
									maxBatchSize, firstResult);
					for (final Announcement announcement : announcements) {
						final List<String> organizations = new ArrayList<String>();
						organizations.add(announcement.getTargetOrganization()
								.getOrgNumber());

						// Construct the AnnouncePatientRequest object
						final AnnouncePatientRequest announcePatientRequest = new AnnouncementToAnnouncePatientRequest()
								.transform(announcement);

						final PatientDemographicsQuery query = new PatientDemographicsQuery();
						String icn = announcement.getPatientIen();
                        if (NullChecker.isNullOrEmpty(icn)) {
                            icn = announcement.getPatientConsentDir().getPatientIen();
                        }
						query.setPatientId(icn);
						/*
						 * PatientDemographicsResponse
						 * patientDemographicsResponse = pdqService
						 * .getPatientDemographics(query); PatientDemographics
						 * demogprahics = patientDemographicsResponse
						 * .getPatientDemographics();
						 * patientType.setLastName(demogprahics.getLastName());
						 * patientType
						 * .setGivenName(demogprahics.getFirstName());
						 * patientType.setSsn(demogprahics.getSsn());
						 */
						final PatientType patientType = new PatientType();
						patientType.setPatientId(icn);
						announcePatientRequest.setPatient(patientType);

						final UserType userType = new UserType();
						userType.setUserId(userName);
						userType.setFacilityNumber(facility);
						announcePatientRequest.setUser(userType);

						announcement.setScheduledTs(new Date());
						this.announcer.storeAnnouncement(announcement);
						// Call asynchronous announce
						// TODO: Fix Transaction Issues
						// Call announce Async. only after the store because the
						// other thread also updates the same record.
						this.patientAnnouncer
								.announceAsync(announcePatientRequest);
						
					}
					
					firstResult++;
					
				} while ((null != announcements)
						&& (announcements.size() == BatchAnnouncerMessageConsumer.MAX_BATCH_SIZE));
			}
		} catch (final Throwable t) {
			throw new Exception(t);
		}
	}

	/**
	 * @see MessageListener#onMessage(Message)
	 */
	@Override
	public void onMessage(final Message message) {
		// Unpack the message
		final ObjectMessage msg = (ObjectMessage) message;
		try {
			final BatchAnnounceType announcement = (BatchAnnounceType) msg
					.getObject();
			final int maxBatchSize = announcement.getMaxBatchSize();
			this.announce(
					announcement.getBatchIds(),
					announcement.getUser(),
					announcement.getFacility(),
					maxBatchSize < 1 ? BatchAnnouncerMessageConsumer.MAX_BATCH_SIZE
							: maxBatchSize);
		} catch (final Exception e) {
			throw new RuntimeException(e);
		}
	}

	@Resource
	public void setContext(final MessageDrivenContext context) {
	}

}
