/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package gov.va.nvap.server.service.partner;

import gov.va.nvap.partner.Partner;
import gov.va.nvap.server.auth.ServiceAuthentication;
import gov.va.nvap.service.auth.ServiceAuthenticationException;
import gov.va.nvap.service.partner.PartnerOnboardingException;
import gov.va.nvap.svc.consentmgmt.PIPInterface;
import gov.va.nvap.svc.consentmgmt.stub.adapter.announce.data.AnnouncerInterface;
import gov.va.nvap.svc.consentmgmt.stub.adapter.announce.data.PatientConsentBatchAnnounce;
import gov.va.nvap.svc.consentmgmt.stub.data.Organization;
import javax.annotation.Resource;

import javax.ejb.Stateless;
import javax.interceptor.Interceptors;
import javax.jws.WebService;
import javax.xml.ws.WebServiceContext;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.ApplicationContext;
import org.springframework.ejb.interceptor.SpringBeanAutowiringInterceptor;

/**
 * 
 * @author vhaislegberb
 */
@Stateless
@WebService(endpointInterface = "gov.va.nvap.service.partner.PartnerOnboardingService")
@Interceptors(SpringBeanAutowiringInterceptor.class)
public class PartnerOnboardingBean implements PartnerOnboardingBeanLocal {

	@Autowired
	private AnnouncerInterface announcer;

	@Autowired
	private PatientConsentBatchAnnounce batchAnnouncer;

	@Autowired
	private PIPInterface policyInformationPoint;
        
        @Autowired
        private ApplicationContext applicationContext;
        @Resource
        private WebServiceContext context;
        private ServiceAuthentication auth = new ServiceAuthentication();
	@Override
	public void addPartner(final Partner partner, final Boolean announce)
			throws PartnerOnboardingException {
                try{
                    auth.checkAuth(context,applicationContext,"addPartner");
                }catch (ServiceAuthenticationException ex) {
                    return;
                }
		try {
			final Organization org = new Organization();
			org.setOrgCommunityIdPrefix(partner.getCommunityIdPrefix());
			org.setOrgContact(partner.getContact());
			org.setOrgDomain(partner.getDomain());
			org.setOrgName(partner.getName());
			org.setOrgNumber(partner.getNumber());
			org.setOrgOid(partner.getOid());
			org.setOrgPhoneNumber(partner.getPhoneNumber());
			org.setActive('Y');

			// persist the organization.
			this.policyInformationPoint.storeOrganization(org);

			// potentially you could do more onboarding stuff here.

			if (announce) {
				// do a targeted batch announce to the new partner
				// TODO: Validate SYSTEM user as the automated on-boarding batch
				// processor
				final String batchId = this.announcer.createAnnouncementsBatch(
						org, "SYSTEM", null, null, true);
				if (null != batchId) {
					this.batchAnnouncer.batchAnnounce(new String[] { batchId },
							"SYSTEM", "PartnerOnboardingBean");
				}
			}

		} catch (final Exception ex) {
			throw new PartnerOnboardingException(
					"Error adding the partner organization.", ex);
		}
	}

	public void setAnnouncer(final AnnouncerInterface announcer) {
            try{
                auth.checkAuth(context,applicationContext,"setAnnouncer");
            }catch (ServiceAuthenticationException ex) {
                return;
            }
            this.announcer = announcer;
	}

	/**
	 * 
	 * @param batchAnnouncer
	 */
	public void setBatchAnnouncer(
			final PatientConsentBatchAnnounce batchAnnouncer) {
            try{    
            auth.checkAuth(context,applicationContext,"setBatchAnnouncer");
            }catch (ServiceAuthenticationException ex) {
                return;
            }
		this.batchAnnouncer = batchAnnouncer;
	}

	@Override
	public void setPartnerActiveFlag(final Boolean activate, final String oid)
			throws PartnerOnboardingException {
            try{
                auth.checkAuth(context,applicationContext,"setPartnerActiveFlag");
            }catch (ServiceAuthenticationException ex) {
                return;
            }	
            try {
			final Organization org = this.policyInformationPoint
					.getOrganizationByOid(oid);
			if (org == null) {
				throw new PartnerOnboardingException(
						String.format(
								"The Organization with OID: %s does not exist in the system.",
								oid));
			}
			org.setActive((activate) ? 'Y' : 'N');

			this.policyInformationPoint.storeOrganization(org);
		} catch (final PartnerOnboardingException poe) {
			throw poe;
		} catch (final Exception ex) {
			throw new PartnerOnboardingException(
					"Error setting Partner active flag.", ex);
		}
	}

	/**
	 * 
	 * @param policyInformationPoint
	 */
	public void setPolicyInformationPoint(
			final PIPInterface policyInformationPoint) {
                try{
                    auth.checkAuth(context,applicationContext,"setPolicyInformationPoint");
                }catch (ServiceAuthenticationException ex) {
                    return;
                }	
		this.policyInformationPoint = policyInformationPoint;
	}
}
