package gov.va.nvap.server.service.privacy;

//import gov.va.med.nhin.adapter.audit.PageInfoType;
//import gov.va.nvap.common.sql.SqlUtil;
import gov.va.nvap.common.sql.SqlUtil;
import gov.va.nvap.common.validation.Assert;
import gov.va.nvap.common.validation.NullChecker;
import gov.va.nvap.privacy.ConsentDirectiveDetailedRequest;
import gov.va.nvap.server.endpoint.psim.PersonServiceInterface;
//import gov.va.nvap.server.service.privacy.emailer.Emailer;
import gov.va.nvap.server.service.privacy.pdf.PdfGenerator;
import gov.va.nvap.service.privacy.ConsentDirectiveService;
import gov.va.nvap.service.pdq.PatientDemographics;
import gov.va.nvap.service.pdq.PatientDemographicsQuery;
import gov.va.nvap.service.pdq.PatientDemographicsResponse;
import gov.va.nvap.service.pdq.PdqService;
import gov.va.nvap.service.pdq.PdqException;
//import gov.va.nvap.privacy.ConsentType;
//import gov.va.nvap.privacy.ConsentDirectiveConfigurationReferenceType;
import gov.va.nvap.privacy.ConsentDirectiveConfigurationRequest;
import gov.va.nvap.privacy.ConsentDirectiveConfigurationResponse;
import gov.va.nvap.privacy.ConsentDirectiveLetterResponse;
//import gov.va.nvap.svc.consentmgmt.stub.data.ConsentDirective;
//import gov.va.nvap.svc.consentmgmt.stub.data.ExpiringConsentConfiguration;
import gov.va.nvap.svc.consentmgmt.PIPInterface;
import java.io.ByteArrayOutputStream;

import org.dozer.Mapper;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Required;
import org.springframework.context.ApplicationContext;
//import org.quartz.Scheduler;
//import org.quartz.CronTrigger;

import java.io.InputStream;
import java.util.ArrayList;
import java.util.Date;
//import java.util.Collection;
//import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.logging.Level;
import java.util.logging.Logger;
//import java.util.Map;
import java.util.Scanner;
import javax.persistence.EntityManager;
import javax.persistence.PersistenceContext;
import javax.persistence.Query;
import javax.persistence.TemporalType;
import org.hibernate.CacheMode;
import org.hibernate.FlushMode;
//import javax.persistence.Query;
//import javax.persistence.TemporalType;

/**
 *
 * @author Zack Peterson
 */
public class ConsentDirectiveServiceImpl implements ConsentDirectiveService {
    
    private static final Logger logger = Logger
			.getLogger(ConsentDirectiveServiceImpl.class.getName());
	
    private PIPInterface pip;
    private Mapper mapper;
    private PersonServiceInterface personService;
    private PdqService pdqService;

    @Autowired
    private ApplicationContext appContext;

    /**
     * Entity Manager instance.
     */
    @PersistenceContext
    private EntityManager em;

    @Override
    public ConsentDirectiveConfigurationResponse
        getConfiguration(ConsentDirectiveConfigurationRequest configRequest) {
            
        return new ConsentDirectiveConfigurationResponse();
    }

    @Override
    public void updateConfiguration(ConsentDirectiveConfigurationRequest configRequest) {
    }

    @Override
    public void sendReport(ConsentDirectiveConfigurationRequest configRequest) {
    }

    @Override
    public ConsentDirectiveLetterResponse getReportLetters(ConsentDirectiveDetailedRequest configRequest) {
        List<AuditedConsentEx> patients = getPatients(configRequest.getStartDate(),
            configRequest.getEndDate(), configRequest.getSsn(), configRequest.getFirstName(),
            configRequest.getLastName(), configRequest.getFacilityName(),
            configRequest.getVisnName(), configRequest.getConsentType(),
            configRequest.getInactivationReason(), configRequest.getEnteredBy(),
            0, 0, null, "ASC", configRequest.getTestPatients());

        final ArrayList<HashMap<String, String>> eHXAuthorizations = new ArrayList<HashMap<String, String>>();
        final ArrayList<HashMap<String, String>> eHXRevocations = new ArrayList<HashMap<String, String>>();
        final ArrayList<HashMap<String, String>> ssaAuthorizations = new ArrayList<HashMap<String, String>>();
        final ArrayList<HashMap<String, String>> ssaRevocations = new ArrayList<HashMap<String, String>>();
        final ArrayList<HashMap<String, String>> eHXRestrictionAuthorizations = new ArrayList<HashMap<String, String>>();
        final ArrayList<HashMap<String, String>> eHXRestrictionRevocations = new ArrayList<HashMap<String, String>>();

        for (final AuditedConsentEx patient : patients) {
            if (NullChecker.isNotEmpty(patient)) {
                final HashMap<String, String> resultMap = new HashMap<String, String>();

                PatientDemographicsQuery demographicsQuery = new PatientDemographicsQuery();
                demographicsQuery.setPatientId(patient.getPatientId());

                PatientDemographicsResponse demographicsResponse;
                try {
                    demographicsResponse = this.pdqService.getPatientDemographics(demographicsQuery);
                    final PatientDemographics demographics = demographicsResponse.getPatientDemographics();

                    if (demographics != null) {
                        resultMap.put("[lastName]", demographics.getLastName());
                        resultMap.put("[firstName]", demographics.getFirstName());
                        resultMap.put("[middleName]", demographics.getMiddleName());

                        final String streetAddressLine1 = demographics.getStreetAddressLine1();
                        final String streetAddressLine2 = demographics.getStreetAddressLine2();
                        final String streetAddressLine3 = demographics.getStreetAddressLine3();
                        final String city = demographics.getResidenceCity();
                        final String state = demographics.getResidenceState();
                        final String zip = demographics.getResidenceZip4();
                        String patientAddress = streetAddressLine1;
                        if (streetAddressLine2 != null && !streetAddressLine2.equals("")) {
                            patientAddress += " " + streetAddressLine2;
                        }
                        if (streetAddressLine3 != null && !streetAddressLine3.equals("")) {
                            patientAddress += " " + streetAddressLine3;
                        }
                        patientAddress += ", " + city + ", " + state + " " + zip;

                        resultMap.put("[patientAddress]", patientAddress);

                        // This will be changed to an actual address when available
                        resultMap.put("[facilityAddress]", demographics.getFacilityName());
                    } else {
                        resultMap.put("[lastName]", "[lastName]");
                        resultMap.put("[firstName]", "[firstName]");
                        resultMap.put("[middleName]", "[middleName]");
                        resultMap.put("[patientAddress]", "[patientAddress]");

                        // This will be changed to an actual address when available
                        resultMap.put("[facilityAddress]", "VA Office");
                    }
                } catch (PdqException e) {
                    ConsentDirectiveServiceImpl.logger.log(Level.INFO, "Error getting patient demographics: %s", e);
                } catch (Exception e) {
                    ConsentDirectiveServiceImpl.logger.log(Level.INFO, "Error getting patient demographics: %s", e);
                }
                
                String consentType = patient.getConsentType();
                if (consentType.equals("NwHIN Authorization")) {
                    eHXAuthorizations.add(resultMap);
                } else if (consentType.equals("NwHIN Revocation")) {
                    eHXRevocations.add(resultMap);
                } else if (consentType.equals("SSA Authorization")) {
                    ssaAuthorizations.add(resultMap);
                } else if (consentType.equals("SSA Revocation")) {
                    ssaRevocations.add(resultMap);
                }  else if (consentType.equals("NwHIN Organization Restriction Authorization")) {
                    eHXRestrictionAuthorizations.add(resultMap);
                }  else if (consentType.equals("NwHIN Organization Restriction Revocation")) {
                    eHXRestrictionRevocations.add(resultMap);
                } else {
                    System.out.println("ConsentDirectiveServiceImpl.java: " + consentType);
                }
            }
        }

        String[] templateArray =  new String[6];
        InputStream inputStream;
        
        inputStream = getClass().getClassLoader().getResourceAsStream("letter_template_ehx_auth.txt");
        templateArray[0] = new Scanner(inputStream, "UTF-8").useDelimiter("\\A").next();

        inputStream = getClass().getClassLoader().getResourceAsStream("letter_template_ehx_revoke.txt");
        templateArray[1] = new Scanner(inputStream, "UTF-8").useDelimiter("\\A").next();

        inputStream = getClass().getClassLoader().getResourceAsStream("letter_template_ssa_auth.txt");
        templateArray[2] = new Scanner(inputStream, "UTF-8").useDelimiter("\\A").next();

        inputStream = getClass().getClassLoader().getResourceAsStream("letter_template_ssa_revoke.txt");
        templateArray[3] = new Scanner(inputStream, "UTF-8").useDelimiter("\\A").next();

        inputStream = getClass().getClassLoader().getResourceAsStream("letter_template_ehx_restrict_auth.txt");
        templateArray[4] = new Scanner(inputStream, "UTF-8").useDelimiter("\\A").next();

        inputStream = getClass().getClassLoader().getResourceAsStream("letter_template_ehx_restrict_revoke.txt");
        templateArray[5] = new Scanner(inputStream, "UTF-8").useDelimiter("\\A").next();
        
        ArrayList<ArrayList<HashMap<String, String>>> patientDetailsList = new ArrayList<ArrayList<HashMap<String, String>>>();
        
        patientDetailsList.add(eHXAuthorizations);
        patientDetailsList.add(eHXRevocations);
        patientDetailsList.add(ssaAuthorizations);
        patientDetailsList.add(ssaRevocations);
        patientDetailsList.add(eHXRestrictionAuthorizations);
        patientDetailsList.add(eHXRestrictionRevocations);

        PdfGenerator pdfGenerator = new PdfGenerator();
        ByteArrayOutputStream pdfStream = pdfGenerator.create(templateArray, patientDetailsList);

        byte[] byteArray = pdfStream.toByteArray();
        
        ConsentDirectiveLetterResponse response = new ConsentDirectiveLetterResponse();
        response.setDownloadByteArray(byteArray);

        return response;
    }

    public final List<AuditedConsentEx> getPatients(final Date startDate,
        final Date endDate, final String ssn, final String firstName,
        final String lastName, final String facilityName,
        final String visnName, final String consentType,
        final String inactivationReason, final String enteredBy,
        final int currentPage,
        final int pageSize, final String sortValue,
        final String sortDirection, final Boolean includeTestPatients) {

        final StringBuffer whereClause = new StringBuffer();
        final StringBuffer sortClause = new StringBuffer();
        final HashMap<String, Object> setParams = new HashMap<String, Object>();

        if (startDate != null) {
            this.appendWhereClause(whereClause, "a.timeOfEvent >= :startDate");
            setParams.put("startDate", startDate);
        }

        if (endDate != null) {
            this.appendWhereClause(whereClause, "a.timeOfEvent <= :endDate");
            setParams.put("endDate", endDate);
        }

        if (!NullChecker.isNullOrEmpty(ssn)) {
            this.appendWhereClause(whereClause, "a.patientSsn = :patientSsn");
            setParams.put("patientSsn", ssn);
        }

        // Task #411
        if (!NullChecker.isNullOrEmpty(firstName)) {
            this.appendWhereClause(whereClause,
                "upper(a.patientGivenName) = :firstName");
            setParams.put("firstName", firstName.toUpperCase());
        }

        // Task #411
        if (!NullChecker.isNullOrEmpty(lastName)) {
            this.appendWhereClause(whereClause,
                "upper(a.patientLastName) = :lastName");
            setParams.put("lastName", lastName.toUpperCase());
        }

        // task 300
        if (!NullChecker.isNullOrEmpty(facilityName)) {
            if (facilityName.equalsIgnoreCase("Unknown")) {
                this.appendWhereClause(whereClause, "a.facilityName IS NULL");
            } else {
                this.appendWhereClause(whereClause, "a.facilityName = :facilityName");
                setParams.put("facilityName", facilityName);
            }
        }

        if (!NullChecker.isNullOrEmpty(visnName)) {
            if (visnName.equalsIgnoreCase("Unknown")) {
                this.appendWhereClause(whereClause, "a.visnName IS NULL");
            } else {
                this.appendWhereClause(whereClause, "a.visnName = :visnName");
                setParams.put("visnName", visnName);
            }
        }

        if (!NullChecker.isNullOrEmpty(consentType)) {
            if (consentType.equalsIgnoreCase("allAuthorizations")) {
                //or statement
                this.appendWhereClause(whereClause, "a.actionType = :actionType");
                setParams.put("actionType", "OPT-IN");
            } else if (consentType.equalsIgnoreCase("allRevocations")) {
                //or statement
                this.appendWhereClause(whereClause, "a.actionType = :actionType");
                setParams.put("actionType", "OPT-OUT");
            } else if (consentType.equalsIgnoreCase("NwHINAuthorizations")) {
                this.appendWhereClause(whereClause, "a.consentType = :consentType");
                setParams.put("consentType", "NwHIN Authorization");
            } else if (consentType.equalsIgnoreCase("SSAAuthorizations")) {
                this.appendWhereClause(whereClause, "a.consentType = :consentType");
                setParams.put("consentType", "SSA Authorization");
            } else if (consentType.equalsIgnoreCase("NwHINRevocations")) {
                this.appendWhereClause(whereClause, "a.consentType = :consentType");
                setParams.put("consentType", "NwHIN Revocation");
            } else if (consentType.equalsIgnoreCase("SSARevocations")) {
                this.appendWhereClause(whereClause, "a.consentType = :consentType");
                setParams.put("consentType", "SSA Revocation");
            } else if (consentType.equalsIgnoreCase("NwHINRestrictions")) {
                this.appendWhereClause(whereClause, "a.consentType = :consentType");
                setParams.put("consentType", "NwHIN Organization Restriction Authorization");
            } else if (consentType.equalsIgnoreCase("NwHINRestrictionRevocations")) {
                this.appendWhereClause(whereClause, "a.consentType = :consentType");
                setParams.put("consentType", "NwHIN Organization Restriction Revocation");
            } else {
                //TODO throw an error
                //program did not receive expected value
            }
        }

        if (!NullChecker.isNullOrEmpty(inactivationReason)) {
            this.appendWhereClause(whereClause,
                "a.optoutReason = :optoutReason");
            setParams.put("optoutReason", inactivationReason);
        }

        if (!NullChecker.isNullOrEmpty(enteredBy)) {
            if (enteredBy.equalsIgnoreCase("eBenefits")) {
                this.appendWhereClause(whereClause, "a.userId = :userId");
                setParams.put("userId", enteredBy);
            } else { //it is employee
                //not eBenefits
                this.appendWhereClause(whereClause, "a.userId <> :userId");
                setParams.put("userId", "eBenefits");
            }
        }

        if (!Boolean.TRUE.equals(includeTestPatients)) {
            this.appendWhereClause(whereClause, SqlUtil.generateExcludeTestPatientsClause(true, "a", "patientSsn"));
        }

        if (!NullChecker.isNullOrEmpty(sortValue)
            && !NullChecker.isNullOrEmpty(sortDirection)) {
            if ("timeOfEvent".equals(sortValue)
                || "createdDate".equals(sortValue)) {
                sortClause.append("a." + sortValue + " " + sortDirection.toUpperCase());
            } else {
                sortClause
                    .append("upper(a." + sortValue + ") " + sortDirection.toUpperCase())
                    .append(", a.timeOfEvent DESC");
            }
        } else {
            sortClause.append("a.timeOfEvent DESC"); //default sort
        }

        final Query query = this.makeAuditedConsentQuery(whereClause,
            sortClause);
        // + " ORDER BY a.timeOfEvent DESC");

        for (final Map.Entry<String, Object> entry : setParams.entrySet()) {
            String pKey = entry.getKey();
            Object pValue = entry.getValue();
            if (pKey.equalsIgnoreCase("startDate") || pKey.equalsIgnoreCase("endDate")) {
                query.setParameter(pKey, (Date) pValue, TemporalType.TIMESTAMP);
            } else {
                query.setParameter(pKey, pValue);
            }
        }

        final Query updatedQuery = this.setAdditionalQueryParamaters(query);
        return this.getResults(updatedQuery, currentPage, pageSize);
    }

    private Query makeAuditedConsentQuery(final StringBuffer whereClause,
        final StringBuffer sortClause) {
        if (NullChecker.isNullOrEmpty(sortClause)) {
            return this.em.createQuery("SELECT a FROM AuditedConsentEx "
                + whereClause.toString());
        } else {
            return this.em.createQuery("SELECT a FROM AuditedConsentEx a "
                + whereClause.toString() + " ORDER BY "
                + sortClause.toString());
        }
    }
    public final List<AuditedConsentEx> getResults(final Query query,
        final int currentPage, final int pageSize) {
        Assert.assertNotEmpty(query, "Query cannot be empty");
        List<AuditedConsentEx> auditedEventEntityList;
        if (pageSize > 0) {
            auditedEventEntityList = query
                .setFirstResult(currentPage * pageSize)
                .setMaxResults(pageSize).getResultList();
        } else {
            auditedEventEntityList = query.getResultList();
        }
        
        return auditedEventEntityList;
    }

    private Query setAdditionalQueryParamaters(final Query theQuery) {
        theQuery.setHint("org.hibernate.cacheable", Boolean.TRUE)
            .setHint("org.hibernate.fetchSize", Integer.valueOf(50))
            .setHint("org.hibernate.cacheMode", CacheMode.GET)
            .setHint("org.hibernate.readOnly", Boolean.TRUE)
            .setHint("org.hibernate.flushMode", FlushMode.MANUAL);
        return theQuery;
    }

    private void appendWhereClause(final StringBuffer buffer, final String str) {
        this.appendWhereClause(buffer, str, " WHERE ", " AND ");
    }

    private void appendWhereClause(final StringBuffer buffer, final String str,
        final String empty, final String notEmpty) {
        if (buffer.length() > 0) {
            buffer.append(notEmpty);
        } else {
            buffer.append(empty);
        }
        buffer.append(str);
    }

    public PIPInterface getPip() {
        return this.pip;
    }

    /**
     * Set the entity manager (Injected by Spring).
     */
    public void setEntityManager(final EntityManager em) {
        this.em = em;
    }

    @Required
    public void setPersonService(PersonServiceInterface personService) {
        this.personService = personService;
    }

    @Required
    public void setPdqService(PdqService pdqService) {
        this.pdqService = pdqService;
    }

    @Required
    public void setPip(PIPInterface pip) {
        this.pip = pip;
    }

    @Required
    public void setMapper(final Mapper mapper) {
        this.mapper = mapper;
    }
	
}