package gov.va.nvap.server.service.privacy;

import gov.va.nvap.privacy.ConsentDirectiveDetailedRequest;
import gov.va.nvap.privacy.ConsentDirectiveConfigurationRequest;
import gov.va.nvap.privacy.ConsentDirectiveConfigurationResponse;
import gov.va.nvap.privacy.ConsentDirectiveLetterResponse;
import gov.va.nvap.server.auth.ServiceAuthentication;
import gov.va.nvap.service.auth.ServiceAuthenticationException;
import gov.va.nvap.service.privacy.ConsentDirectiveService;
import javax.annotation.Resource;

import javax.ejb.Remote;
import javax.ejb.Stateless;
import javax.ejb.TransactionAttribute;
import javax.ejb.TransactionAttributeType;
import javax.interceptor.Interceptors;
import javax.jws.WebMethod;
import javax.jws.WebParam;
import javax.jws.WebResult;
import javax.jws.WebService;
import javax.jws.soap.SOAPBinding;
import javax.xml.ws.BindingType;
import javax.xml.ws.WebServiceContext;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.context.ApplicationContext;
import org.springframework.ejb.interceptor.SpringBeanAutowiringInterceptor;

@SuppressWarnings("all")
@WebService(portName = "ConsentDirectivePort", name = "ConsentDirectivePortType",
        serviceName = "ConsentDirectiveService", targetNamespace = "http://gov.va.nvap.service.privacy/",
        wsdlLocation = "META-INF/xml-resources/web-services/ConsentManagementService/wsdl/ConsentDirectiveService.wsdl")
@BindingType("http://schemas.xmlsoap.org/wsdl/soap/http")
@SOAPBinding(parameterStyle = SOAPBinding.ParameterStyle.BARE)
@Stateless(name = "ConsentDirectiveService", mappedName = "ConsentDirectiveService")
@Remote(ConsentDirectiveService.class)
@Interceptors(SpringBeanAutowiringInterceptor.class)
@TransactionAttribute(TransactionAttributeType.REQUIRED)
public class ConsentDirectiveServicePort implements ConsentDirectiveService {

    /**
     * Inject Spring bean, so that we have all dependencies injected.
     */
    @Autowired
    @Qualifier(value = "ConsentDirectiveService")
    private ConsentDirectiveService consentDirectiveService;
    @Autowired
    private ApplicationContext applicationContext;
    @Resource
    private WebServiceContext context;
    private ServiceAuthentication auth = new ServiceAuthentication();

    public ConsentDirectiveServicePort() {
    }

    /**
     * @param consentDirectiveConfigurationRequest
     * @return returns gov.va.nvap.privacy.
     * ConsentDirectiveConfigurationResponseType
     */
    @WebMethod(operationName = "GetConfiguration")
    @WebResult(name = "ConsentDirectiveConfigurationResponse", targetNamespace = "http://gov.va.nvap.service.privacy/", partName = "ConsentDirectiveConfigurationResponse")
    public ConsentDirectiveConfigurationResponse getConfiguration(
            @WebParam(name = "ConsentDirectiveConfigurationRequest", targetNamespace = "http://gov.va.nvap.service.privacy/", partName = "ConsentDirectiveConfigurationRequest") final ConsentDirectiveConfigurationRequest consentDirectiveConfigurationRequest) {
        try{
            auth.checkAuth(context, applicationContext, "getConfiguration");
        }catch (ServiceAuthenticationException ex) {
             return null;
        }
        try {
            return this.consentDirectiveService
                    .getConfiguration(consentDirectiveConfigurationRequest);
        } catch (final Exception ex) {
            // TODO: Exception handling

            return null;
        }
    }

    @WebMethod(operationName = "UpdateConfiguration")
    public void updateConfiguration(
            @WebParam(name = "ConsentDirectiveConfigurationRequest", targetNamespace = "http://gov.va.nvap.service.privacy/", partName = "ConsentDirectiveConfigurationRequest") final ConsentDirectiveConfigurationRequest consentDirectiveConfigurationRequest) {
        try{
            auth.checkAuth(context, applicationContext, "updateConfiguration");
        }catch (ServiceAuthenticationException ex) {
            return;
        }
        try {
            this.consentDirectiveService
                    .updateConfiguration(consentDirectiveConfigurationRequest);
        } catch (final Exception ex) {
            // TODO: Exception handling	
        }
    }

    @WebMethod(operationName = "SendReport")
    public void sendReport(
            @WebParam(name = "ConsentDirectiveConfigurationRequest", targetNamespace = "http://gov.va.nvap.service.privacy/", partName = "ConsentDirectiveConfigurationRequest") final ConsentDirectiveConfigurationRequest consentDirectiveConfigurationRequest) {
        try{
            auth.checkAuth(context, applicationContext, "sendReport");
        }catch (ServiceAuthenticationException ex) {
                return;
        }
        try {
            this.consentDirectiveService
                    .sendReport(consentDirectiveConfigurationRequest);
        } catch (final Exception ex) {
            // TODO: Exception handling	
        }
    }

    @WebMethod(operationName = "GetReportLetters")
    @Override
    @WebResult(name = "ConsentDirectiveLetterResponse", targetNamespace = "http://gov.va.nvap.service.privacy/", partName = "ConsentDirectiveLetterResponse")
    public ConsentDirectiveLetterResponse getReportLetters(
            @WebParam(name = "ConsentDirectiveRequest", targetNamespace = "http://gov.va.nvap.service.privacy/", partName = "ConsentDirectiveRequest") final ConsentDirectiveDetailedRequest consentDirectiveRequest) {
        try{
            auth.checkAuth(context, applicationContext, "getReportLetters");
        }catch (ServiceAuthenticationException ex) {
            return null;
        }
        try {
            return this.consentDirectiveService
                    .getReportLetters(consentDirectiveRequest);
        } catch (final Exception ex) {
            // TODO: Exception handling
            return null;
        }
    }
}