package gov.va.nvap.svc.consentmgmt.stub.data;

import java.io.Serializable;
import java.util.Date;

import javax.persistence.Basic;
import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.FetchType;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.persistence.JoinColumn;
import javax.persistence.ManyToOne;
import javax.persistence.NamedNativeQueries;
import javax.persistence.NamedNativeQuery;
import javax.persistence.Table;
import javax.persistence.Temporal;
import javax.persistence.TemporalType;

/**
 *
 * @author Zack Peterson
 * @author vhaislegberb
 */
@Entity
@Table(name = "PATIENT_CONSENT_DIR")
@NamedNativeQueries({
    @NamedNativeQuery(
            name="FacilityOptInConsent.findAuthorizedConsentInformation",
            query=FacilityOptInConsent.findExpiringConsentDirectivesForSummaryNativeSQL,
            resultClass=FacilityOptInConsent.class
    )
})

public class FacilityOptInConsent implements Serializable {
	private static final long serialVersionUID = 1L;

    protected static final String findExpiringConsentDirectivesForSummaryNativeSQL=
    "select" +
            " CONSENT_DIR_ID, " +
            " PATIENT_SSN, " +
    "    OPTIN_CONSENT_TYPE_ID," +
    "    OPTIN_DATE," +
    "    FACILITY_NAME " +
    "from (" +
    "    select" +
    "        X.*," +
    "        ROW_NUMBER() over (partition by PATIENT_IEN, OPTIN_CONSENT_TYPE_ID order by OPTIN_DATE DESC) as RN" +
    "    from (" +
    "        select" +
            " C.CONSENT_DIR_ID, " +
            " J.PATIENT_SSN, " +
    "            C.PATIENT_IEN," +
    "            C.OPTIN_CONSENT_TYPE_ID," +
    "            J.OPTIN_DATE," +
    "            J.FACILITY_NAME" +
    "        from PATIENT_CONSENT_DIR C inner join (" +
    "            select " +
    "                A.PATIENT_ID, " +
                    "A.PATIENT_SSN," +
    "                A.CONSENT_TYPE," +
    "                A.FACILITY_NAME," +
    "                A.CREATED_DATE as OPTIN_DATE," +
    "                B.CONSENT_TYPE_ID" +
    "            from" +
    "                CONSENT_AUDIT A left join CONSENT_TYPE B ON A.CONSENT_TYPE = B.NAME" +
    "            where" +
    "                ACTION_TYPE = 'OPT-IN'" +
    "        ) J" +
    "        on C.PATIENT_IEN = J.PATIENT_ID" +
    "        where" +
    "            OPTIN_CONSENT_TYPE_ID is not null and" +
    "            OPTIN_CONSENT_TYPE_ID = CONSENT_TYPE_ID and" +
    "            OPTOUT_DATE is null and" +
    "            EXPIRATION_DATE > systimestamp" +
    "    ) X" +
    ")" +
    " where RN = 1";

	@Id
	@GeneratedValue(strategy = GenerationType.AUTO)
	@Basic(optional = false)
	@Column(name = "CONSENT_DIR_ID")
	private Long consentDirId;
    @Column(name = "PATIENT_SSN")
	private String patientSsn;
	@ManyToOne(fetch = FetchType.EAGER)
	@JoinColumn(name = "OPTIN_CONSENT_TYPE_ID")
	private ConsentType optinConsentType;
	@Basic(optional = false)
	@Column(name = "OPTIN_DATE")
	@Temporal(TemporalType.TIMESTAMP)
	private Date optinDate;
    
    // Demographics info from CONSENT_AUDIT
    @Column(name = "FACILITY_NAME")
    private String facilityName;

    public String getFacilityName() {
        return facilityName;
    }

    public void setFacilityName(String facilityName) {
        this.facilityName = facilityName;
    }

    public String getPatientSsn() {
        return patientSsn;
    }

    public void setPatientSsn(String patientSsn) {
        this.patientSsn = patientSsn;
    }

	public FacilityOptInConsent() {
	}

	public FacilityOptInConsent(final Long consentDirId) {
		this.consentDirId = consentDirId;
	}

	public FacilityOptInConsent(final Long consentDirId, final String patientIen,
			final Date optinDate, final Date expirationDate,
			final Date optinTS, final ConsentType consentType) {
		this.consentDirId = consentDirId;
		this.optinDate = optinDate;
		this.optinConsentType = consentType;
		}

	@Override
	public boolean equals(final Object object) {
		// TODO: Warning - this method won't work in the case the id fields are
		// not set
		if (!(object instanceof FacilityOptInConsent)) {
			return false;
		}
		final FacilityOptInConsent other = (FacilityOptInConsent) object;
		if (((this.consentDirId == null) && (other.consentDirId != null))
				|| ((this.consentDirId != null) && !this.consentDirId
						.equals(other.consentDirId))) {
			return false;
		}
		return true;
	}

	public Long getConsentDirId() {
		return this.consentDirId;
	}

	public Date getOptinDate() {
		return this.optinDate;
	}

	@Override
	public int hashCode() {
		int hash = 0;
		hash += (this.consentDirId != null ? this.consentDirId.hashCode() : 0);
		return hash;
	}

	public void setConsentDirId(final Long consentDirId) {
		this.consentDirId = consentDirId;
	}

	public void setOptinDate(final Date optinDate) {
		this.optinDate = optinDate;
	}

	@Override
	public String toString() {
		return "gov.va.med.vler.nhin.vap.pip.entities.DetailedConsentDirective[consentDirId="
				+ this.consentDirId + "]";
	}

	public void setOptinConsentType(ConsentType optinConsentType) {
		this.optinConsentType = optinConsentType;
	}

	public ConsentType getOptinConsentType() {
		return optinConsentType;
	}
}
