package gov.va.nvap.web.announce;

import gov.va.nvap.common.validation.NullChecker;
import gov.va.nvap.web.app.ResponseDispatcherHttpServlet;
import gov.va.nvap.web.helper.privacy.ConsentManagementHelper;
import gov.va.nvap.web.util.Paginator;
import gov.va.nvap.web.util.date.DateUtil;

import java.io.IOException;
import java.util.Calendar;
import java.util.Date;

import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpSession;

/**
 * @author Tim Vu / Asha Amritraj
 */
public class BatchManagerQuery extends ResponseDispatcherHttpServlet {

	/**
	 * Serial UID.
	 */
	private static final long serialVersionUID = 8860158686782275698L;

	public ConsentManagementHelper getCmsHelper() {
		final ConsentManagementHelper cmsHelper = this.getBean("cmsHelper",
				ConsentManagementHelper.class);
		return cmsHelper;
	}

	public void search(final HttpServletRequest request,
			final HttpServletResponse response) throws ServletException,
			IOException {
		final HttpSession session = request.getSession(false);
		final String startDate = request.getParameter("startDate");
		final String endDate = request.getParameter("endDate");
		final int recordsPerPage = Integer.parseInt(request
				.getParameter("recordsPerPage"));
		final String remoteOrg = request.getParameter("remoteOrganization");

		if (this.validate(request, response, "search")) {
			Date sd = (Date) session.getAttribute("batchManagerQueryStartDate");
			Date ed = (Date) session.getAttribute("batchManagerQueryEndDate");
			if (sd == null) {
				if (!NullChecker.isNullOrEmpty(startDate)) {
					sd = DateUtil.parseDateMMddyyyy(startDate);
					sd = DateUtil.voidTime(sd);
					session.setAttribute("batchManagerQueryStartDate", sd);
				}
			}

			if (ed == null) {
				if (!NullChecker.isNullOrEmpty(endDate)) {
					ed = DateUtil.parseDateMMddyyyy(endDate);
					ed = DateUtil.voidTime(ed);
                    ed = DateUtil.addTime(ed, Calendar.HOUR_OF_DAY, 23);
                    ed = DateUtil.addTime(ed, Calendar.MINUTE, 59);
                    ed = DateUtil.addTime(ed, Calendar.SECOND, 59);                    
					session.setAttribute("batchManagerQueryEndDate", ed);
				}
			}
			session.setAttribute("batchManagerQueryRemoteOrganization",
					remoteOrg);
			session.setAttribute("paginator", new Paginator(recordsPerPage));
			this.forward(request, response, "success");
		} else {
			this.forward(request, response, "validate");
		}
	}

	@Override
	protected void unspecified(final HttpServletRequest request,
			final HttpServletResponse response) throws ServletException,
			IOException {
		final HttpSession session = request.getSession(false);

		session.removeAttribute("batchManagerQueryStartDate");
		session.removeAttribute("batchManagerQueryEndDate");
		session.removeAttribute("paginator");
		session.removeAttribute("batchManagerQueryRemoteOrganization");
		session.removeAttribute("batchManagerResults");

		request.setAttribute("organizations", this.getCmsHelper()
				.getAllowedNonConsumerOnlyOrganizations());

		this.forward(request, response, "show");
	}

}
