/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package gov.va.nvap.web.app;

import gov.va.nvap.common.validation.NullChecker;

import java.io.IOException;
import java.io.PrintStream;
import java.io.PrintWriter;
import java.io.StringWriter;

import javax.servlet.Filter;
import javax.servlet.FilterChain;
import javax.servlet.FilterConfig;
import javax.servlet.ServletException;
import javax.servlet.ServletRequest;
import javax.servlet.ServletResponse;
import javax.servlet.http.HttpServletResponse;

/**
 * A filter to process errors.
 * 
 * @author David Vazquez
 */
public class PermissionAppFilter implements Filter {

	public static String getStackTrace(final Throwable t) {
		String stackTrace = null;
		try {
			final StringWriter sw = new StringWriter();
			final PrintWriter pw = new PrintWriter(sw);
			t.printStackTrace(pw);
			pw.close();
			sw.close();
			stackTrace = sw.getBuffer().toString();
		} catch (final Exception ex) {
		}
		return stackTrace;
	}

	public PermissionAppFilter() {
	}

	@Override
	public void destroy() {
	}

	private void doAfterProcessing(final ServletRequest request,
			final ServletResponse response) throws IOException,
			ServletException {
	}

	private boolean doBeforeProcessing(final ServletRequest request,
			final ServletResponse response) throws IOException,
			ServletException {
		// Do not cache any pages
		boolean ret;
		final HttpServletResponse resp = (HttpServletResponse) response;
		resp.setHeader("Cache-control", "no-cache");
		resp.setHeader("Expires", "0");
		ret = true;
		return ret;
	}

	@Override
	public void doFilter(final ServletRequest request,
			final ServletResponse response, final FilterChain chain)
			throws IOException, ServletException {

		if (this.doBeforeProcessing(request, response)) {
			Throwable problem = null;
			try {
				chain.doFilter(request, response);
			} catch (final Throwable t) {
				// If an exception is thrown somewhere down the filter chain,
				// we still want to execute our after processing, and then
				// rethrow the problem after that.
				problem = t;
				t.printStackTrace();
			}

			this.doAfterProcessing(request, response);

			// If there was a problem, we want to rethrow it if it is
			// a known type, otherwise log it.
			if (problem != null) {
				if (problem instanceof ServletException) {
					throw (ServletException) problem;
				}
				if (problem instanceof IOException) {
					throw (IOException) problem;
				}
				this.sendProcessingError(problem, response);
			}
		}
	}

	/**
	 * Init method for this filter
	 */
	@Override
	public void init(final FilterConfig filterConfig) {
	}

	private void sendProcessingError(final Throwable t,
			final ServletResponse response) {
		final String stackTrace = PermissionAppFilter.getStackTrace(t);

		if (NullChecker.isNotEmpty(stackTrace)) {
			try {
				response.setContentType("text/html");
				final PrintStream ps = new PrintStream(
						response.getOutputStream());
				final PrintWriter pw = new PrintWriter(ps);
				pw.print("<html>\n<head>\n<title>Error</title>\n</head>\n<body>\n"); // NOI18N

				// PENDING! Localize this for next official release
				pw.print("<h1>The resource did not process correctly</h1>\n<pre>\n");
				pw.print(stackTrace);
				pw.print("</pre></body>\n</html>"); // NOI18N
				pw.close();
				ps.close();
				response.getOutputStream().close();
			} catch (final Exception ex) {
			}
		} else {
			try {
				final PrintStream ps = new PrintStream(
						response.getOutputStream());
				t.printStackTrace(ps);
				ps.close();
				response.getOutputStream().close();
			} catch (final Exception ex) {
			}
		}
	}

	@Override
	public String toString() {
		final StringBuffer sb = new StringBuffer("PermissionAppFilter()");
		return (sb.toString());
	}

}
