package gov.va.nvap.web.patient;

import gov.va.nvap.common.transformer.TransformerException;
import gov.va.nvap.common.validation.Assert;
import gov.va.nvap.common.validation.NullChecker;
import gov.va.nvap.common.xpath.XPathException;
import gov.va.nvap.privacy.ConsentDirectiveOptOutReasonType;
import gov.va.nvap.privacy.ConsentDirectiveReferenceType;
import gov.va.nvap.privacy.ConsentType;
import gov.va.nvap.privacy.OrganizationType;
import gov.va.nvap.service.adapter.doc.AdapterException;
import gov.va.nvap.service.pdq.Facility;
import gov.va.nvap.service.pdq.PatientCorrelationsQuery;
import gov.va.nvap.service.pdq.PatientCorrelationsResponse;
import gov.va.nvap.service.pdq.PatientDemographics;
import gov.va.nvap.service.pdq.PatientDemographicsQuery;
import gov.va.nvap.service.pdq.PatientDemographicsResponse;
import gov.va.nvap.service.pdq.PdqException;
import gov.va.nvap.service.pdq.PdqService;
import gov.va.nvap.service.pdq.RemovePatientCorrelationRequest;
import gov.va.nvap.svc.consentmgmt.stub.adapter.announce.data.Announcement;
import gov.va.nvap.svc.consentmgmt.stub.adapter.announce.data.PatientAnnouncer;
import gov.va.nvap.web.consent.comment.ConsentComment;
import gov.va.nvap.web.consent.comment.ConsentCommentDAO;
import gov.va.nvap.web.patient.PatientComment;
import gov.va.nvap.web.patient.PatientCommentDAO;
import gov.va.nvap.web.helper.document.DocumentHelper;
import gov.va.nvap.web.helper.document.MediaType;
import gov.va.nvap.web.helper.document.RepresentationType;
import gov.va.nvap.web.helper.facility.FacilityHelper;
import gov.va.nvap.web.helper.privacy.ConsentManagementHelper;
import gov.va.nvap.web.util.Constants;
import gov.va.nvap.web.util.DASUtil;
import gov.va.nvap.web.util.date.DateUtil;
import gov.va.nvap.web.util.file.FileUploadUtil;
import gov.va.nvap.web.consent.audit.dao.AuditedConsentDAO;
import gov.va.nvap.web.consent.audit.AuditedConsentEx;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.Reader;
import java.io.UnsupportedEncodingException;
import java.net.URL;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.ejb.EJB;
import javax.servlet.ServletException;
import javax.servlet.ServletOutputStream;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpSession;
import org.apache.commons.fileupload.FileItem;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.json.JSONException;
import org.json.JSONObject;
import weblogic.net.http.HttpsURLConnection;

import java.util.logging.Level;
import java.util.logging.Logger;

/**
 * Get the detailed view of the patient.
 *
 * @author 
 * @author                  
 * @author 
 */
public class PatientDetails extends
		gov.va.nvap.web.app.ResponseDispatcherHttpServlet {
	static private final Log LOG = LogFactory.getLog(PatientDetails.class);
    
	/**
	 * Serial UID.
	 */
	private static final long serialVersionUID = 960355389572259403L;
    
    private static final int bufferSize = 8192;

	/**
	 * Announce patients to the NwHIN.
	 */
	@EJB(beanInterface = PatientAnnouncer.class, mappedName = "PatientAnnouncer")
	private PatientAnnouncer patientAnnouncer;
	/**
	 * Patient Demographics Service to get the demographics and correlations
	 * from the MPI.
	 */
	@EJB(beanInterface = PdqService.class, mappedName = "PdqService")
	PdqService pdqService;

	/**
	 * Announce is called when the user clicked on Announce on the Patient
	 * Details page.
     * 
     * @param request http servlet request
     * @param response http servlet response
     * 
     * @throws javax.servlet.ServletException
     * @throws java.io.IOException
	 */
	public void announce(final HttpServletRequest request,
			final HttpServletResponse response) throws ServletException,
			IOException {
		// Get the ICN
		final String icn = request.getParameter("icn");
		Assert.assertNotEmpty(icn, "ICN cannot be null!");
		if (this.validate(request, response, "announce")) {
            PatientAnnounceThread announceThread = 
                    new PatientAnnounceThread("announcerThread");
            announceThread.setCmsHelper(this.getCmsHelper());
            announceThread.setFacilityHelper(this.getFacilityHelper());
            announceThread.setPatientAnnouncer(this.patientAnnouncer);
            announceThread.setPatientIcn(icn);
            announceThread.setPdqService(this.pdqService);
            announceThread.setRemoteUserId(request.getRemoteUser());
            announceThread.start();
            
            final HttpSession session = request.getSession(false);
            session.setAttribute("announcementSent", "true");
            
            this.forward(request, response, "search");
		} else {
			// Validation
			this.forward(request, response, "validate");
		}
	}

	/**
	 * Get the consent management helper from Spring.
	 */
	public ConsentManagementHelper getCmsHelper() {
		final ConsentManagementHelper cmsHelper = this.getBean("cmsHelper",
				ConsentManagementHelper.class);
		return cmsHelper;
	}

	/**
	 * Get the document helper from Spring.
	 */
	public DocumentHelper getDocumentHelper() {
		return this.getBean("adapterDocumentHelper", DocumentHelper.class);
	}

	/**
	 * Get the facility helper from Spring.
	 */
	public FacilityHelper getFacilityHelper() {
		final FacilityHelper facilityHelper = this.getBean("facilityHelper",
				FacilityHelper.class);
		return facilityHelper;
	}
        
        public ConsentCommentDAO getConsentCommentDAO() {
            final ConsentCommentDAO consentCommentDAO = this.getBean("consentCommentDAO", ConsentCommentDAO.class);
            
            return consentCommentDAO;
        }
        
        public PatientCommentDAO getPatientCommentDAO() {
            final PatientCommentDAO patientCommentDAO = this.getBean("patientCommentDAO", PatientCommentDAO.class);
            
            return patientCommentDAO;
        }
        
        public AuditedConsentDAO getAuditedConsentDAO() {
            final AuditedConsentDAO auditedConsentDAO = this.getBean("auditedConsentDAO", AuditedConsentDAO.class);
            
            return auditedConsentDAO;
        }
        
	/**
	 * Show the consent management history.
     * 
     * @param request http servlet request
     * @param response http servlet response
     * 
     * @throws javax.servlet.ServletException
     * @throws java.io.IOException
	 */
	public void history(final HttpServletRequest request,
			final HttpServletResponse response) throws ServletException,
			IOException {
		final HttpSession session = request.getSession(false);
		// Document type = OPT-IN/OPT-OUT
		final String documentType = request.getParameter("documentType");
		final String consentDirId = request.getParameter("consentDirId");

		Assert.assertNotEmpty(documentType, "Document Type cannot be null!");
		final String document = this.getCmsHelper().retrieveDocument(
				consentDirId, documentType);
		// Set the document for further queries
		session.setAttribute("document", document);
		session.setAttribute("documentType", documentType);
		session.setAttribute("consentDirId", consentDirId);
		// Return the PDF
		try {
			if ( DASUtil.isDASDocument(document)){
				// Get from DAS and show
				if ( Constants.isDasUseMockRetrieve() ){
					mockGetDocumentFromDAS(response.getOutputStream());
				}else{
					getDocumentFromDAS(response.getOutputStream(), document);
				}
			}else{			
				final String attachment = this.getDocumentHelper()
						.getPrivacyConsentDirectiveAttachment(document);
				if (NullChecker.isNotEmpty(attachment)) {
					this.getDocumentHelper()
							.writeAttachmentToStream(
									response,
									attachment,
									MediaType.fromValue(this.getDocumentHelper()
											.getPrivacyConsentDirectiveMediaType(
													document)),
									RepresentationType.B64);
				} else {
					session.setAttribute("informationMessage",
							"There are no attachments associated with this consent directive.");
					this.forward(request, response, "noattachment");
				}
			}
		} catch (final TransformerException ex) {
			throw new ServletException(ex);
		} catch (final XPathException ex) {
			throw new ServletException(ex);
		}

	}

	/**
	 * User selected opt-in in the Patient details page.
     * 
     * @param fileItems list of uploaded file items
     * @param request http servlet request
     * @param response http servlet response
     * 
     * @throws javax.servlet.ServletException
     * @throws java.io.IOException
	 */
	public void optIn(final List<FileItem> fileItems,
			final HttpServletRequest request, final HttpServletResponse response)
			throws ServletException, IOException {
		final HttpSession session = request.getSession(false);

		// Get the fields
		final Map<String, String> formFields = FileUploadUtil
				.getFormFields(fileItems);

		// Get the uploaded files
		final List<FileItem> filesUploaded = FileUploadUtil
				.getFilesUploaded(fileItems);
                
                String consentId = "";
                
		// Get the icn
		final String icn = formFields.get("icn");
		final String consentType = formFields.get("consentType");
		final String purposeOfUse = formFields.get("purposeOfUse");
		final String expirationYears = formFields.get("expirationYears");
                final String comments = formFields.get("comments");
		// Get the organizations selected by the user
		final String newRight = formFields.get("newRight");
		// Get the date signed
		final String signatureDateStr = formFields.get("signatureDate");
		String userFacilityStationId = formFields.get("userFacility");

		String[] organizationNumbers = null;
		if (NullChecker.isNotEmpty(newRight)) {
			organizationNumbers = newRight.split(",");

		}

		if (this.validate(formFields, request, response, "optIn")) {
			// Get the date after validation
			final Date signatureDate = DateUtil
					.parseDateMMddyyyy(signatureDateStr);
			// Maybe go to MPI
			final PatientDemographics patientDemographics = (PatientDemographics) session
					.getAttribute("patientDemographics");
			// Get the files uploaded
			final String userId = request.getRemoteUser();
			// Do it as part of the GUI if possible

			if (NullChecker.isEmpty(userFacilityStationId)) {
				userFacilityStationId = this.getFacilityHelper()
						.getFaciltyStationIdByUserId(userId);
			}
			if ("NwHIN Organization Restriction Modification"
					.equals(consentType)) {
				this.getCmsHelper().optOut(
						icn,
						patientDemographics,
						filesUploaded,
						"Revoked",
						userId,
						userFacilityStationId,
						signatureDate,
						ConsentType.NW_HIN_ORGANIZATION_RESTRICTION_REVOCATION
								.value(), purposeOfUse);
				consentId = this.getCmsHelper()
						.optIn(icn,
								organizationNumbers,
								patientDemographics,
								filesUploaded,
								userId,
								userFacilityStationId,
								signatureDate,
								ConsentType.NW_HIN_ORGANIZATION_RESTRICTION_AUTHORIZATION
										.value(), purposeOfUse, expirationYears);

			} else {
				// Call opt-in with files uploaded
				consentId = this.getCmsHelper().optIn(icn, organizationNumbers,
						patientDemographics, filesUploaded, userId,
						userFacilityStationId, signatureDate, consentType,
						purposeOfUse, expirationYears);
			}
                        
                        //handle comments
                        if(!NullChecker.isEmpty(comments)) {
                            ConsentComment comment = new ConsentComment();
                            comment.setComments(comments);
                            comment.setConsentType(consentType);
                            comment.setUserId(userId);
                            comment.setDateAdded(new Date());
                            comment.setConsentId(Long.valueOf(consentId));
                            
                            this.getConsentCommentDAO().create(comment);
                        }
                            
                        // Remove after opt-in
			// Success
			this.forward(request, response, "success");
			// announce(request, response);
		} else {
			// Validate
			this.forward(request, response, "validate");
		}
	}

	/**
	 * The user selected opt-out from the Patient details page.
     * 
     * @param request http servlet request
     * @param response http servlet response
     * 
     * @throws javax.servlet.ServletException
     * @throws java.io.IOException
	 */
	public void optOut(final List<FileItem> fileItems,
			final HttpServletRequest request, final HttpServletResponse response)
			throws ServletException, IOException {
		final HttpSession session = request.getSession(false);

		// Get the fields
		final Map<String, String> formFields = FileUploadUtil
				.getFormFields(fileItems);
		// Get the uploaded files
		final List<FileItem> filesUploaded = FileUploadUtil
				.getFilesUploaded(fileItems);
		final String consentType = formFields.get("consentType");
                
                String consentId = "";
                
		// Get the icn
		final String icn = formFields.get("icn");
		// Get the opt out reason
		final String reason = formFields.get("reason");
		String signatureDateStr = formFields.get("signatureDate");
                String comments = formFields.get("comments");
		String deceasedDateStr = formFields.get("deceasedDate");
		String userFacilityStationId = formFields.get("userFacility");
		final String purposeOfUse = formFields.get("purposeOfUse");
		final String expiryYears = formFields.get("expirationYears");
		if (ConsentDirectiveOptOutReasonType.PATIENT_DECEASED.value().equals(
				reason)) {
			signatureDateStr = deceasedDateStr;
		}

		if (this.validate(formFields, request, response, "optOut")) {
			final Date signatureDate = DateUtil
					.parseDateMMddyyyy(signatureDateStr);
			// Maybe go to MPI
			final PatientDemographics patientDemographics = (PatientDemographics) session
					.getAttribute("patientDemographics");
			// Get the files uploaded
			final String userId = request.getRemoteUser();
			if (NullChecker.isEmpty(userFacilityStationId)) {
				userFacilityStationId = this.getFacilityHelper()
						.getFaciltyStationIdByUserId(userId);
			}

			// Opt-out without files
			consentId = this.getCmsHelper().optOut(icn, patientDemographics, filesUploaded,
					reason, userId, userFacilityStationId, signatureDate,
					consentType, purposeOfUse);
			if (ConsentDirectiveOptOutReasonType.NEW_AUTHORIZATION.value()
					.equals(reason)) {
				if (ConsentType.NW_HIN_REVOCATION.value().equals(consentType)) {
					consentId = this.getCmsHelper().optIn(icn, null, patientDemographics,
							null, userId, userFacilityStationId, signatureDate,
							ConsentType.NW_HIN_AUTHORIZATION.value(),
							purposeOfUse, expiryYears);
				} else if (ConsentType.SSA_REVOCATION.value().equals(
						consentType)) {
					consentId = this.getCmsHelper().optIn(icn, null, patientDemographics,
							null, userId, userFacilityStationId, signatureDate,
							ConsentType.SSA_AUTHORIZATION.value(),
							purposeOfUse, expiryYears);
				}
			}
                        
                        //handle comments
                        if(!NullChecker.isEmpty(comments)) {
                            ConsentComment comment = new ConsentComment();
                            comment.setComments(comments);
                            comment.setConsentType(consentType);
                            comment.setUserId(userId);
                            comment.setDateAdded(new Date());
                            comment.setConsentId(Long.valueOf(consentId));
                            
                            this.getConsentCommentDAO().create(comment);
                        }
                        
			// Remove attributes before exiting
			// Success
			this.forward(request, response, "success");
		} else {
			// Validate
			this.forward(request, response, "validate");
		}
	}

	/**
	 * If a system administrator needs to remove all correlations from the MPI.
     * 
     * @param request http servlet request
     * @param response http servlet response
     * 
     * @throws javax.servlet.ServletException
     * @throws java.io.IOException
	 */
	public void revoke(final HttpServletRequest request,
			final HttpServletResponse response) throws ServletException,
			IOException {
		// Get the icn
		final String icn = request.getParameter("icn");
                final String comments = request.getParameter("comments");
                final String consentType = request.getParameter("consentType");
		if (this.validate(request, response, "revoke")) {
			// Make the correlations query
			final PatientCorrelationsQuery correlationsQuery = new PatientCorrelationsQuery();
			correlationsQuery.setPatientId(icn);
			try {
				// Get the correlations response
				final PatientCorrelationsResponse correlationsResponse = this.pdqService
						.getCorrelatedFacilities(correlationsQuery);
				// Get the demographics
				final PatientDemographicsQuery demographicsQuery = new PatientDemographicsQuery();
				demographicsQuery.setPatientId(icn);
				final PatientDemographicsResponse demographicsResponse = this.pdqService
						.getPatientDemographics(demographicsQuery);
				final PatientDemographics demographics = demographicsResponse
						.getPatientDemographics();
				// Get all the facilities
				final List<Facility> facilities = correlationsResponse
						.getFacilities();
				// For each NwHIN facilities other than DoD, create a request to
				// remove the correlations on the MPI.
				if (NullChecker.isNotEmpty(facilities)) {
					for (final Facility facility : facilities) {
						// Get the organization
						final OrganizationType stdOrganization = this
								.getCmsHelper().getOrganizationByNumber(
										facility.getFacilityId());
						// You really cant remove the DoD correlations
						if (NullChecker.isEmpty(stdOrganization)
								|| (NullChecker.isNotEmpty(stdOrganization) && stdOrganization
										.getOrgNumber().equals("200DOD"))) {
						} else {
							// Get the facility to be removed from the MPI
							// correlation
							String correlatedAssigningAuthorityName = facility
									.getAssigningAuthorityName();
							final String correlatedAssigningAuthorityOid = stdOrganization
									.getOrgOid();
							final String firstName = demographics
									.getFirstName();
							final String middleName = demographics
									.getMiddleName();
							final String lastName = demographics.getLastName();
							final RemovePatientCorrelationRequest removePatientCorrelationRequest = new RemovePatientCorrelationRequest();
							if (NullChecker
									.isEmpty(correlatedAssigningAuthorityName)) {
								correlatedAssigningAuthorityName = stdOrganization
										.getOrgNumber();
							}
							// Set values
							removePatientCorrelationRequest
									.setCorrelatedAssigningAuthorityName(correlatedAssigningAuthorityName);
							removePatientCorrelationRequest
									.setCorrelatedAssigningAuthorityOid(correlatedAssigningAuthorityOid);
							removePatientCorrelationRequest
									.setCorrelatedPatientId(facility
											.getCorrelatedPatientId());
							removePatientCorrelationRequest.setPatientId(icn);
							removePatientCorrelationRequest
									.setPatientFirstName(firstName);
							removePatientCorrelationRequest
									.setPatientMiddleName(middleName);
							removePatientCorrelationRequest
									.setPatientLastName(lastName);
							removePatientCorrelationRequest.setUserId(request
									.getRemoteUser());
							// Call the service to remove correlations.
							RemovePatientCorrelationRequest r = this.pdqService
									.removePatientCorrelation(removePatientCorrelationRequest);
                                                        
                                                        //handle comments
                                                        if(!NullChecker.isEmpty(comments)) {
                                                            ConsentComment comment = new ConsentComment();
                                                            comment.setComments(comments);
                                                            comment.setConsentType(consentType);
                                                            comment.setUserId(request.getRemoteUser());
                                                            comment.setDateAdded(new Date());
                                                            //TODO: comment.setConsentId(Long.valueOf(consentId));

                                                            this.getConsentCommentDAO().create(comment);
                                                        }
						}
					}
				}
			} catch (final PdqException ex) {
				throw new ServletException(ex);
			}
			this.forward(request, response, "success");
		} else {
			this.forward(request, response, "validate");
		}
	}

    /**
     * Base method that handles requests that don't specify another method
     * 
     * @param request http servlet request
     * @param response http servlet response
     * 
     * @throws javax.servlet.ServletException
     * @throws java.io.IOException
     */
	@Override
	protected void unspecified(final HttpServletRequest request,
			final HttpServletResponse response) throws ServletException,
			IOException {
		final HttpSession session = request.getSession(false);

		final String icn = (String) session.getAttribute("icn");
        
		if (!NullChecker.isNullOrEmpty(icn)) {
			// Store the request id in the session
			this.setRequestId(request, response);
			try {
                // Get consent directives for current patient
                List<ConsentDirectiveReferenceType> consentDirectiveReferences =
                        this.getCmsHelper().getConsentDirectiveHistory(icn);
                
                // Get ID numbers for applicable consent directives
                List<Long> consentIds = new ArrayList<Long>();
                for (ConsentDirectiveReferenceType consent : consentDirectiveReferences) {
                    if (!consentIds.contains(Long.parseLong(consent.getConsentDirId()))) {
                        consentIds.add(Long.parseLong(consent.getConsentDirId()));
                    }
                }
                
                //now that we have the consent Ids for this patient, let's get the comments for them
                if (consentIds == null || consentIds.isEmpty()) {
                    session.setAttribute("comments", null);
                } else {
                    List<ConsentComment> comments = this.getConsentCommentDAO().getCommentsByConsentIds(consentIds);
                    session.setAttribute("comments", comments);
                }
                
                // get CONSENT_AUDIT info for this patient
                String patientId = consentDirectiveReferences.get(0).getPatientIen();
                List<Object> auditedConsentsUnordered = this.getAuditedConsentDAO().getResultsByPatientId(patientId);
                
                // make a copy of auditedConsentsUnordered so that the copy will match the consentDirectiveReferences
                // according to the UserIds in auditedConsentsUnordered
                // but first make sure there is one auditedConsent for each consentDirectiveReferences
                ArrayList<AuditedConsentEx> auditedConsents = new ArrayList<AuditedConsentEx>();
                int cdrSize = consentDirectiveReferences.size();
                for (ConsentDirectiveReferenceType cdr : consentDirectiveReferences) {
                    // find the userId in auditedConsentsUnordered for this consentDirectiveReference
                    for (int i = 0; i < auditedConsentsUnordered.size(); i++) {
                        AuditedConsentEx ac = (AuditedConsentEx)auditedConsentsUnordered.get(i);
                        String consentType = ac.getConsentType();
                        String cdrOptInConsentType = 
                            (cdr.getOptinConsentType() != null) ? cdr.getOptinConsentType().value() : "null";
                        String cdrOptOutConsentType = 
                            (cdr.getOptoutConsentType() != null) ? cdr.getOptoutConsentType().value() : "null";

                        // see if the consentTypes match between auditedConsentsUnordered and consentDirectiveReference
                        if (consentType.equals(cdrOptInConsentType) || 
                            consentType.equals(cdrOptOutConsentType)) {
                            // we found a match based on the consent type, so add this auditedConsent to the
                            // new list and remove it from auditedConsentsUnordered
                            auditedConsents.add(ac);
                            auditedConsentsUnordered.remove(i);
                            i = auditedConsentsUnordered.size() + 1;
                        }
                    }
                }
                
                session.setAttribute("auditedConsents", auditedConsents);
                
                //                   11/25/2014 JAZZ 442142 - 
                // Get the list of announcements for this patient by consent directive 
                // ID list so that we can change caption of "Announce" button to "Re-Announce" 
                // if the patient has already been announced.
                String lastAnnounceMessage = "";
                if (consentIds.isEmpty()) {
                    session.setAttribute("hasBeenAnnounced", false);
                } else {
                    List<Announcement> announcements = this.patientAnnouncer.getAnnouncementsByConsentDirectiveId(consentIds);
                    session.setAttribute("hasBeenAnnounced", !announcements.isEmpty());

                    if (!announcements.isEmpty()) {
                        // Check to see if this patient was announced successfully.
                        for (int i = announcements.size() - 1; i >= 0; i--) {
                            if (announcements.get(i).getAnnouncementResult() != null) {
                                if (Integer.parseInt(announcements.get(i).getAnnouncementResult()) > 0) {
                                    if (announcements.get(i).getCompletedTs() != null) {
                                        lastAnnounceMessage = "Last successful announcement: " + 
                                            new SimpleDateFormat("yyyy-MM-dd HH:mm:ss").format(announcements.get(i).getCompletedTs());
                                        break;
                                    }
                                }
                            }
                        }
                        if (lastAnnounceMessage.length() == 0) {
                            // There are no successful announcements, let's show the last failed announcement, if any.
                            for (int i = announcements.size() - 1; i >= 0; i--) {
                                if (announcements.get(i).getAnnouncementResult() != null) {
                                    if (announcements.get(i).getCompletedTs() != null) {
                                        lastAnnounceMessage = "No successful announcements. Last failed announcement: " +
                                            new SimpleDateFormat("yyyy-MM-dd HH:mm:ss").format(announcements.get(i).getCompletedTs());
                                        break;
                                    }
                                }
                            }
                        }
                        if (lastAnnounceMessage.length() == 0) {
                            // There are no completed announcements, let's show the last scheduled announcement, if any.
                            for (int i = announcements.size() - 1; i >= 0; i--) {
                                if (announcements.get(i).getCreatedTs()!= null) {
                                    lastAnnounceMessage = "No completed announcements. Last scheduled announcement: " +
                                        new SimpleDateFormat("yyyy-MM-dd HH:mm:ss").format(announcements.get(i).getCreatedTs());
                                    break;
                                }
                            }
                        }
                    }
                }
                session.setAttribute("lastAnnouncement", lastAnnounceMessage);
                
				// Get the demographic details section
				final PatientDemographicsQuery patientDemographicsQuery = new PatientDemographicsQuery();
				patientDemographicsQuery.setPatientId(icn);
				final PatientDemographicsResponse patientDemographicsResponse = this.pdqService
						.getPatientDemographics(patientDemographicsQuery);
				final PatientDemographics demographics = patientDemographicsResponse
						.getPatientDemographics();
				session.setAttribute("patientDemographics", demographics);
				session.setAttribute("isOrganizationExclusionsAllowed", this
						.getCmsHelper().isOrganizationExclusionsAllowed());
				session.setAttribute("isFormUploadAllowed", this.getCmsHelper()
						.isFormUploadEnabled());

				session.setAttribute("maxFileUploadSize", this
						.getDocumentHelper().getMaxUploadFileSize());
				// Get the allowed organizations to list them on the patient
				// details page
				final Collection<OrganizationType> allowedOrganizations = this
						.getCmsHelper().getAllowedNonConsumerOnlyOrganizations();
				session.setAttribute("allowedOrganizations",
						allowedOrganizations);
				final Collection<gov.va.nvap.svc.facility.data.Facility> allowedFacilities = this
						.getFacilityHelper().getAllowedVistAFacilities();
				session.setAttribute("allowedFacilities", allowedFacilities);
				// Get the Default Facility by user id
				/*
				final gov.va.nvap.svc.facility.data.Facility defaultUserFacility = this
						.getFacilityHelper().getFacilityStationByVHAUserId(
								request.getRemoteUser());
				if (NullChecker.isNotEmpty(defaultUserFacility)) {
					session.setAttribute("defaultUserFacility",
							defaultUserFacility);
				}
				*/
				final gov.va.nvap.svc.facility.data.Facility defaultUserFacility = 
					(gov.va.nvap.svc.facility.data.Facility) session.getAttribute("defaultUserFacility");
				
				// Get all the consent directives - needed to populate the
				// consent management history
//				final List<ConsentDirectiveReferenceType> consentDirectiveReferences = this
//						.getCmsHelper().getConsentDirectiveHistory(icn);
				// Get the active consent directive to display before history
				ConsentDirectiveReferenceType activeConsentDirectiveReferenceType = null;
				ConsentDirectiveReferenceType ssaActiveConsentDirectiveReferenceType = null;
				ConsentDirectiveReferenceType nwhinOrgRestrictionsActiveConsentDirectiveReferenceType = null;
				if (NullChecker.isNotEmpty(consentDirectiveReferences)) {
					for (final ConsentDirectiveReferenceType type : consentDirectiveReferences) {
						if (NullChecker.isEmpty(type.getOptoutTS())
								&& ConsentType.NW_HIN_AUTHORIZATION.value()
										.equals(type.getOptinConsentType()
												.value())
								&& type.getExpirationDate().after(new Date())) {
							activeConsentDirectiveReferenceType = type;
						}
						if (NullChecker.isEmpty(type.getOptoutTS())
								&& ConsentType.SSA_AUTHORIZATION.value()
										.equals(type.getOptinConsentType()
												.value())
								&& type.getExpirationDate().after(new Date())) {
							ssaActiveConsentDirectiveReferenceType = type;
						}
						if (NullChecker.isEmpty(type.getOptoutTS())
								&& ConsentType.NW_HIN_ORGANIZATION_RESTRICTION_AUTHORIZATION
										.value().equals(
												type.getOptinConsentType()
														.value())) {
							nwhinOrgRestrictionsActiveConsentDirectiveReferenceType = type;
						}
					}
				}
				// Set all the consent directives to populate history
				final List<Map<String, Object>> consentDirectiveReferencesMapList = new ArrayList<Map<String, Object>>();
				if (NullChecker.isNotEmpty(consentDirectiveReferences)) {
					for (final ConsentDirectiveReferenceType type : consentDirectiveReferences) {
						final Map<String, Object> consentDirectiveReferencesMap = new HashMap<String, Object>();
						consentDirectiveReferencesMap.put("consentDirId",
								type.getConsentDirId());
						// Get the consent type
						if (NullChecker.isNotEmpty(type.getOptinConsentType())) {
							consentDirectiveReferencesMap.put(
									"optinConsentType", type
											.getOptinConsentType().value());
						}
						if (NullChecker.isNotEmpty(type.getOptoutConsentType())) {
							consentDirectiveReferencesMap.put(
									"optoutConsentType", type
											.getOptoutConsentType().value());
						}
						consentDirectiveReferencesMap.put("purposeOfUse",
								type.getPurposeOfUse());
						consentDirectiveReferencesMap.put("optinTS",
								type.getOptinTS());
						consentDirectiveReferencesMap.put(
								"excludedOrganizations",
								type.getExcludedOrganizations());
						consentDirectiveReferencesMap.put("expirationDate",
								type.getExpirationDate());
						consentDirectiveReferencesMap.put("optinDate",
								type.getOptinDate());
						consentDirectiveReferencesMap.put("optoutDate",
								type.getOptoutDate());
						consentDirectiveReferencesMap.put("optoutTS",
								type.getOptoutTS());
						if (NullChecker.isNotEmpty(type.getOptoutReason())) {
							consentDirectiveReferencesMap.put("optoutReason",
									type.getOptoutReason().value());
						}
						consentDirectiveReferencesMap.put("patientIen",
								type.getPatientIen());

						try {
							final String optinDocument = this.getCmsHelper()
									.retrieveDocument(type.getConsentDirId(),
											"Authorize");
							
							if (DASUtil.isDASDocument(optinDocument)){
								consentDirectiveReferencesMap.put(
										"hasOptinDocumentAttachment",
										NullChecker.isNotEmpty(optinDocument));
							}else{							
								final String optinAttachment = optinDocument == null ? null : this
										.getDocumentHelper()
										.getPrivacyConsentDirectiveAttachment(
												optinDocument);
								consentDirectiveReferencesMap.put(
										"hasOptinDocumentAttachment",
										NullChecker.isNotEmpty(optinAttachment));
							}
							
							if (NullChecker.isNotEmpty(type.getOptoutTS())) {
								final String optoutDocument = this
										.getCmsHelper().retrieveDocument(
												type.getConsentDirId(),
												"Revoke");
								
								if (DASUtil.isDASDocument(optoutDocument)){
									consentDirectiveReferencesMap.put(
											"hasOptoutDocumentAttachment",
											NullChecker.isNotEmpty(optoutDocument));
								}else{
									final String optoutAttachment = optoutDocument == null ? null : this
											.getDocumentHelper()
											.getPrivacyConsentDirectiveAttachment(
													optoutDocument);
									consentDirectiveReferencesMap.put(
											"hasOptoutDocumentAttachment",
											NullChecker
													.isNotEmpty(optoutAttachment));
								}
								
							} else {
								consentDirectiveReferencesMap.put(
										"hasOptoutDocumentAttachment", false);
							}
						} catch (final TransformerException ex) {
							throw new ServletException(ex);
						} catch (final XPathException ex) {
							throw new ServletException(ex);
						}
						consentDirectiveReferencesMapList
								.add(consentDirectiveReferencesMap);
					}
				}
				session.setAttribute("consentDirectiveReferences",
						consentDirectiveReferencesMapList);

				// if there is a active consent directive, then get the excluded
				// organizations
				if (NullChecker
						.isNotEmpty(nwhinOrgRestrictionsActiveConsentDirectiveReferenceType)) {
					final Collection<OrganizationType> excludedOrganizations = nwhinOrgRestrictionsActiveConsentDirectiveReferenceType
							.getExcludedOrganizations();
					final List<OrganizationType> authorizedOrganizations = new ArrayList<OrganizationType>();
					for (final OrganizationType organization : allowedOrganizations) {
						boolean isExcluded = false;
						for (final OrganizationType excludedOrganization : excludedOrganizations) {
							if (organization.getOrgNumber().equals(
									excludedOrganization.getOrgNumber())) {
								isExcluded = true;
							}
						}
						if (!isExcluded) {
							authorizedOrganizations.add(organization);
						}
					}
					// Set in session
					session.setAttribute("excludedOrganizations",
							excludedOrganizations);
					session.setAttribute("authorizedOrganizations",
							authorizedOrganizations);
				}
				// Must be opt-out
				session.setAttribute("consentDirective",
						activeConsentDirectiveReferenceType);
				session.setAttribute("ssaConsentDirective",
						ssaActiveConsentDirectiveReferenceType);

				session.setAttribute("isNewAuthorizationAllowed", true);
				session.setAttribute("isSSANewAuthorizationAllowed", true);
/*
				if (NullChecker.isNotEmpty(activeConsentDirectiveReferenceType)) {
					Calendar dateCal = Calendar.getInstance();
					dateCal.setTime(activeConsentDirectiveReferenceType
							.getExpirationDate());
					dateCal.add(Calendar.DATE, -180);
					Date expirableDate = dateCal.getTime();
					if (expirableDate.before(new Date())) {
						session.setAttribute("isNewAuthorizationAllowed", true);
					}
				}
				if (NullChecker
						.isNotEmpty(ssaActiveConsentDirectiveReferenceType)) {
					Calendar dateCal = Calendar.getInstance();
					dateCal.setTime(ssaActiveConsentDirectiveReferenceType
							.getExpirationDate());
					dateCal.add(Calendar.DATE, -180);
					Date expirableDate = dateCal.getTime();
					if (expirableDate.before(new Date())) {
						session.setAttribute("isSSANewAuthorizationAllowed",
								true);
					}
				}
*/
				session.setAttribute("nwhinOrganizationRestrictionsDirective",
						nwhinOrgRestrictionsActiveConsentDirectiveReferenceType);

				session.setAttribute("optOutReasons", this.getCmsHelper()
						.getOptoutReasons());

				// Get the correlations section
				final PatientCorrelationsQuery patientCorrelationsQuery = new PatientCorrelationsQuery();
				patientCorrelationsQuery.setPatientId(icn);
				// Get the correlations
				final PatientCorrelationsResponse patientCorrelationsResponse = this.pdqService
						.getCorrelatedFacilities(patientCorrelationsQuery);
				final List<Facility> facilities = patientCorrelationsResponse
						.getFacilities();
				final List<Map<String, String>> correlations = new ArrayList<Map<String, String>>();
				if (NullChecker.isNotEmpty(facilities)) {
					for (final Facility facility : facilities) {
						final String correlatedPatientId = facility
								.getCorrelatedPatientId();
						final String correlatedFacilityId = facility
								.getFacilityId();
						final OrganizationType organization = this
								.getCmsHelper().getOrganizationByNumber(
										correlatedFacilityId);

						if (NullChecker.isNotEmpty(organization)) {
							final Map<String, String> correlation = new HashMap<String, String>();
							final String facilityName = organization
									.getOrgName();
							final String facilityNumber = organization
									.getOrgNumber();
							final String orgOid = organization.getOrgOid();
							correlation.put("organizationOid", orgOid);

							correlation.put("facilityName", facilityName);
							final String oid = facility
									.getAssigningAuthorityNumber();
							/*
							 *
							 * if (NullChecker.isNotEmpty(oid)) { final String
							 * aaName = this .getFacilityHelper()
							 * .getAuthorityName(
							 * facility.getAssigningAuthorityNumber()); if
							 * (NullChecker.isNotEmpty(aaName)) {
							 * correlation.put("assigningAuthorityName",
							 * aaName); correlation.put("assigningAuthorityOid",
							 * oid); } }
							 */
							// correlation.put("assigningAuthorityName",
							// aaName);
							correlation.put("assigningAuthorityOid", oid);
							correlation.put("facilityNumber", facilityNumber);
							correlation.put("correlatedPatientId",
									correlatedPatientId);
							correlations.add(correlation);
						}
					}
				}

				// Set the correlations int he session
				session.setAttribute("correlations", correlations);
                                
                                // get the comments for the patient detail section
                                final String patientIen = demographics.getIcn();
                                session.setAttribute("patientIen", patientIen);
                                List<PatientComment> patientComments = this.getPatientCommentDAO().getCommentsByPatientIen(patientIen);
                                session.setAttribute("patientComments", patientComments);
                                
			} catch (final PdqException ex) {
				throw new ServletException(ex);
			}
			// SUCCESS
			this.forward(request, response, "show");
		} else {
			this.forward(request, response, "noshow");
		}
	}

    /**
     * Handles consent directive file upload
     * 
     * @param fileItems list of uploaded file items
     * @param request http servlet request
     * @param response http servlet response
     * 
     * @throws javax.servlet.ServletException
     * @throws java.io.IOException
     */
	public void upload(final List<FileItem> fileItems,
			final HttpServletRequest request, final HttpServletResponse response)
			throws ServletException, IOException {
		// Get the fields
		final Map<String, String> formFields = FileUploadUtil
				.getFormFields(fileItems);
		// Get the uploaded files
		final List<FileItem> filesUploaded = FileUploadUtil
				.getFilesUploaded(fileItems);
		if (NullChecker.isNotEmpty(filesUploaded)) {
			// Get the document type and the consent directive Id
			final String documentType = formFields.get("documentType");
			final String consentDirId = formFields.get("consentDirId");
			final String userId = request.getRemoteUser();
			final String userFacilityStationId = this.getFacilityHelper()
					.getFaciltyStationIdByUserId(userId);
			this.getCmsHelper().updateDocument(consentDirId, documentType,
					filesUploaded, userId, userFacilityStationId);
		}
		this.forward(request, response, "success");
	}

	/**
	 * View the C32 from the adapter. It makes a webservice request to the
	 * adapter, pulls the data from local VistA systems and displays it to the
	 * user.
     * 
     * @param request http servlet request
     * @param response http servlet response
     * 
     * @throws javax.servlet.ServletException
     * @throws java.io.IOException
	 */
	public void view(final HttpServletRequest request,
			final HttpServletResponse response) throws ServletException,
			IOException {
		final HttpSession session = request.getSession(false);

		final String icn = (String) session.getAttribute("icn");
		if (NullChecker.isNotEmpty(icn)) {
			try {
				// Get the C32 document from the adapter
				String document = this.getDocumentHelper().getC32Document(icn,
						request.getRemoteUser());
				if (NullChecker.isEmpty(document)) {
					throw new ServletException(
							"There was an issue retrieving health summary for this patient with ICN:"
									+ icn);
				}
				// Remove xml-stylesheet
				document = document
						.replaceAll(
								"<?xml-stylesheet type=\"text/xsl\" href=\"CCD.xsl\"?>",
								"");
				// Get the style sheets
				final String styleSheetViewType = request
						.getParameter("styleSheetViewType");
				if ("xml".equals(styleSheetViewType)) {
					response.setContentType("text/xml;charset=UTF-8");
					response.getWriter().write(document);
				} else {
					// Update session in the document
					final String html = this.getDocumentHelper().getHtml(
							document, styleSheetViewType);
					session.setAttribute("updatedDocument", html);
					session.setAttribute("document", document);
					session.setAttribute("isXmlViewEnabled", this
							.getDocumentHelper().isXmlViewEnabled());
					this.forward(request, response, "showdoc");
				}
			} catch (final AdapterException ex) {
				throw new ServletException(ex);
			}
		} else {
			this.forward(request, response, "noshow");
		}

	}
        
        public void addComment(final HttpServletRequest request, final HttpServletResponse response) throws ServletException, IOException {
            final String comments = request.getParameter("comments");
            final String consentType = request.getParameter("consentType");
            final long consentId = Long.valueOf(request.getParameter("consentId"));
            final String userId = request.getRemoteUser();

            if(NullChecker.isNotEmpty(comments)) {
                //handle comments
                ConsentComment comment = new ConsentComment();
                comment.setComments(comments);
                comment.setConsentType(consentType);
                comment.setUserId(userId);
                comment.setDateAdded(new Date());
                comment.setConsentId(consentId);

                this.getConsentCommentDAO().create(comment);

                // Success
                this.forward(request, response, "success");
            } else {
                // Validate
                this.forward(request, response, "validate");
            }
        }
        
        public void addPatientComment(final HttpServletRequest request, final HttpServletResponse response) throws ServletException, IOException {
            final String comments = request.getParameter("comments");
            final String patientIen = request.getParameter("patientIen");
            final String userId = request.getRemoteUser();

            if(NullChecker.isNotEmpty(comments)) {
                //handle comments
                PatientComment comment = new PatientComment();
                comment.setUserId(userId);
                comment.setDateAdded(new Date());
                comment.setComments(comments);
                comment.setPatientIen(patientIen);

                this.getPatientCommentDAO().create(comment);

                // Success
                this.forward(request, response, "success");
            } else {
                // Validate
                this.forward(request, response, "validate");
            }
        }
        
	public void authorizeNwHIN(final HttpServletRequest request,
			final HttpServletResponse response) throws ServletException,
			IOException {
		final HttpSession session = request.getSession(false);

		final String icn = (String) session.getAttribute("icn");
		if (NullChecker.isNotEmpty(icn)) {
		} else {
		}
		this.forward(request, response, "authorizeNwHIN");
	}

	public void revokeNwHIN(final HttpServletRequest request,
			final HttpServletResponse response) throws ServletException,
			IOException {
		final HttpSession session = request.getSession(false);

		final String icn = (String) session.getAttribute("icn");
		if (NullChecker.isNotEmpty(icn)) {
		} else {
		}
		this.forward(request, response, "revokeNwHIN");
	}

	public void authorizeSSA(final HttpServletRequest request,
			final HttpServletResponse response) throws ServletException,
			IOException {
		final HttpSession session = request.getSession(false);

		final String icn = (String) session.getAttribute("icn");
		if (NullChecker.isNotEmpty(icn)) {
		} else {
		}
		this.forward(request, response, "authorizeSSA");
	}

	public void revokeSSA(final HttpServletRequest request,
			final HttpServletResponse response) throws ServletException,
			IOException {
		final HttpSession session = request.getSession(false);

		final String icn = (String) session.getAttribute("icn");
		if (NullChecker.isNotEmpty(icn)) {
		} else {
		}
		this.forward(request, response, "revokeSSA");
	}

	public void authorizeNwHINOrganizationRestrictions(
			final HttpServletRequest request, final HttpServletResponse response)
			throws ServletException, IOException {
		final HttpSession session = request.getSession(false);

		final String icn = (String) session.getAttribute("icn");
		if (NullChecker.isNotEmpty(icn)) {
		} else {
		}
		this.forward(request, response,
				"authorizeNwHINOrganizationRestrictions");
	}

	public void revokeNwHINOrganizationRestrictions(
			final HttpServletRequest request, final HttpServletResponse response)
			throws ServletException, IOException {
		final HttpSession session = request.getSession(false);

		final String icn = (String) session.getAttribute("icn");
		if (NullChecker.isNotEmpty(icn)) {
		} else {
		}
		this.forward(request, response, "revokeNwHINOrganizationRestrictions");
	}

	public void modifyNwHINOrganizationRestrictions(
			final HttpServletRequest request, final HttpServletResponse response)
			throws ServletException, IOException {
		final HttpSession session = request.getSession(false);

		final String icn = (String) session.getAttribute("icn");
		if (NullChecker.isNotEmpty(icn)) {
		} else {
		}
		this.forward(request, response, "modifyNwHINOrganizationRestrictions");
	}
	
        public void addCommentForm(final HttpServletRequest request, final HttpServletResponse response) throws ServletException, IOException {
            final HttpSession session = request.getSession(false);

            final String consentId = request.getParameter("consentId");
            final String consentType = request.getParameter("consentType");
            session.setAttribute("consentId", consentId);
            session.setAttribute("consentType", consentType);

            this.forward(request, response, "addCommentForm");
	}
        
        public void addPatientCommentForm(final HttpServletRequest request, final HttpServletResponse response) throws ServletException, IOException {
            final HttpSession session = request.getSession(false);

            final String patientIen = request.getParameter("patientIen");
            session.setAttribute("patientIen", patientIen);

            this.forward(request, response, "addPatientCommentForm");
	}
        
	private void mockGetDocumentFromDAS(ServletOutputStream ostream) throws IOException {
        InputStream is = 
   			 this.getClass().getClassLoader().getResourceAsStream("gov/va/nvap/web/das.txt");
        
        final char[] buffer = new char[bufferSize];
        final StringBuilder out = new StringBuilder();
        try {
            Reader in = new InputStreamReader(is, "UTF-8");
            for (;;) {
            int rsz = in.read(buffer, 0, buffer.length);
            if (rsz < 0)
                break;
            out.append(buffer, 0, rsz);
          }
        }
        catch (UnsupportedEncodingException ex) {
            /* ... */
        }
        catch (IOException ex) {
            /* ... */
        }
        String jsonString = out.toString();
        
        
        ostream.close();
        is.close();
        
        String xmlId = "";
        int xmlLength = 0;
        String pdfId = "";
        int pdfLength = 0;
        try {
            JSONObject jObject = new JSONObject(jsonString);
            xmlId = jObject.getString("gridfsID");
            xmlLength = jObject.getInt("gridfsLength");
            pdfId = ((JSONObject)jObject.getJSONArray("gridfsAttachments").get(0)).getString("gridfsID");
            pdfLength = ((JSONObject)jObject.getJSONArray("gridfsAttachments").get(0)).getInt("gridfsLength");
        } catch (JSONException ex) {
            Logger.getLogger(PatientDetails.class.getName()).log(Level.SEVERE, null, ex);
        }
        
        is = this.getClass().getClassLoader().getResourceAsStream("gov/va/nvap/web/das.pdf");
        byte[] buf = new byte[bufferSize];
        int c = 0;
        while ((c = is.read(buf, 0, buf.length)) > 0) {
        	ostream.write(buf, 0, c);
        	ostream.flush();
        }
    }
 
 
 private void getDocumentFromDAS(ServletOutputStream ostream, String externalDocumentId) throws IOException{
    // Build and send initial DAS request
    String dasRetrieveDocumentEndPoint =  Constants.getDasRetrieveDocumentEndPoint();
    String dasRetrieveOneDocumentEndPoint =  Constants.getDasRetrieveOneDocumentEndPoint();
    String dasRetrieveDocumentUrlWithDocId =  dasRetrieveDocumentEndPoint + externalDocumentId;

    //HttpClient client = HttpClientBuilder.create().build();
    //HttpGet request = new HttpGet(dasRetrieveDocumentUrlWithDocId);
    URL url = new URL(dasRetrieveDocumentUrlWithDocId);
    Logger.getLogger(PatientDetails.class.getName()).log(Level.INFO, "DAS URL: "+dasRetrieveDocumentUrlWithDocId);
    HttpsURLConnection conn = null;
    conn = (HttpsURLConnection) url.openConnection();
    conn.connect();

    //added the response message on failure as well, because it can help with debugging potentially
    if (conn.getResponseCode() != 200) {
        throw new RuntimeException("\nFailed : HTTP error code : " + conn.getResponseCode() + "\nHTTP response : " + conn.getResponseMessage());
    }
    InputStream inputStream =  conn.getInputStream();
    // HttpResponse response = client.execute(request);
    // HttpEntity entity = response.getEntity();
    
    // Read DAS response to JSON string
    if (inputStream != null) {
        
        final char[] buffer = new char[bufferSize];
        final StringBuilder out = new StringBuilder();
        try {
            Reader in = new InputStreamReader(inputStream, "UTF-8");
            for (;;) {
            int rsz = in.read(buffer, 0, buffer.length);
            if (rsz < 0)
                break;
            out.append(buffer, 0, rsz);
          }
        }
        catch (UnsupportedEncodingException ex) {
            Logger.getLogger(PatientDetails.class.getName()).log(Level.SEVERE, null, ex);
        }
        catch (IOException ex) {
            Logger.getLogger(PatientDetails.class.getName()).log(Level.SEVERE, null, ex);
        }
        String jsonString = out.toString();
        inputStream.close();
        
        // Extract eAuth PDF ID from JSON DAS response
        String pdfId = "";
        int pdfLength = 0;
        try {
            JSONObject jObject = new JSONObject(jsonString);
            pdfId = ((JSONObject)jObject.getJSONArray("gridfsAttachments").get(0)).getString("gridfsID");
            pdfLength = ((JSONObject)jObject.getJSONArray("gridfsAttachments").get(0)).getInt("gridfsLength");
        } catch (JSONException ex) {
            Logger.getLogger(PatientDetails.class.getName()).log(Level.SEVERE, null, ex);
        }
        
        if (pdfLength <= 0) {
            Logger.getLogger(PatientDetails.class.getName()).log(Level.WARNING, 
                    "eAuth PDF length is not greater than zero");
        }
        
        dasRetrieveDocumentUrlWithDocId =  dasRetrieveOneDocumentEndPoint + pdfId;
         Logger.getLogger(PatientDetails.class.getName()).log(Level.INFO, "DAS Doc URL: "+dasRetrieveDocumentUrlWithDocId);
        url = new URL(dasRetrieveDocumentUrlWithDocId);
        conn = (HttpsURLConnection) url.openConnection();
        conn.connect();

        //added the response message on failure as well, because it can help with debugging potentially
        if (conn.getResponseCode() != 200) {
            throw new RuntimeException("\nFailed : HTTP error code : " + conn.getResponseCode() + "\nHTTP response : " + conn.getResponseMessage());
        }
        InputStream is =  conn.getInputStream();
        if (is != null) {
            // Print PDF
            byte[] buf = new byte[bufferSize];
            int c;
            while ((c = is.read(buf, 0, buf.length)) > 0) {
                ostream.write(buf, 0, c);
                ostream.flush();
            }
            is.close();
        } else {
            Logger.getLogger(PatientDetails.class.getName()).log(Level.WARNING, "No inner PDF returned");
        }
    } else {
        Logger.getLogger(PatientDetails.class.getName()).log(Level.WARNING, "No documents returned");
    }
 }
}
