package gov.va.nvap.web.patient;

import gov.va.nvap.common.validation.NullChecker;
import gov.va.nvap.svc.facility.data.Facility;
import gov.va.nvap.web.app.ResponseDispatcherHttpServlet;
import gov.va.nvap.web.helper.facility.FacilityHelper;

import java.io.IOException;

import javax.servlet.ServletException;
import javax.servlet.http.Cookie;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpSession;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

/**
 * @author               
 */
public class SetDefaultFacility extends gov.va.nvap.web.app.ResponseDispatcherHttpServlet {
	static private final Log LOG = LogFactory.getLog(SetDefaultFacility.class);
	/**
	 * Serial UID.
	 */
	private static final long serialVersionUID = 5356729716646683218L;
	
	public void setManualFacility(final HttpServletRequest request,
			final HttpServletResponse response) throws ServletException,
			IOException {

		if (this.validate(request, response)) {
			final HttpSession session = request.getSession(false);

			final String newFacility = request.getParameter("facility");
			session.setAttribute("defaultUserFacility", newFacility);
			response.addCookie(setFacilityCookie(newFacility));
			this.forward(request, response, "success");
		}else {
			this.forward(request, response, "validate");
			return;
		}
	}
	
	public void removeManualFacility(final HttpServletRequest request,
			final HttpServletResponse response) throws ServletException,
			IOException {
		if (this.validate(request, response)) {
			final HttpSession session = request.getSession(false);
			gov.va.nvap.svc.facility.data.Facility defaultAutoUserFacility = this.getFacilityHelper().getFacilityStationByVHAUserId(
					request.getRemoteUser());
			if(!NullChecker.isNullOrEmpty(defaultAutoUserFacility)){//there is an automatic
				session.setAttribute("defaultUserFacility", defaultAutoUserFacility);
			}else{// there is no automatic or manual
				session.removeAttribute("defaultUserFacility");
			}
			response.addCookie(removeFacilityCookie());
			this.forward(request, response, "success");
		}else {
			this.forward(request, response, "validate");
			return;
		}
	}
	
	private Cookie removeFacilityCookie() {
		Cookie vapCookie = null;
		vapCookie = new Cookie("facility", "");
		vapCookie.setMaxAge(0);
		return vapCookie;
	}
	
	private Cookie setFacilityCookie(final String setFacility) {

		Cookie vapCookie = null;

		if (setFacility != null) {
			if (setFacility.equals("")) {
				vapCookie = new Cookie("facility", "");
				vapCookie.setMaxAge(0);
			} else {
				vapCookie = new Cookie("facility", setFacility);
				vapCookie.setMaxAge(5 * 365 * 24 * 60 * 60);
			}
		}
		return vapCookie;
	}
	
	/**
	 * Get the facility helper from Spring.
	 */
	public FacilityHelper getFacilityHelper() {
		final FacilityHelper facilityHelper = this.getBean("facilityHelper",
				FacilityHelper.class);
		return facilityHelper;
	}
	
	@Override
	protected void unspecified(final HttpServletRequest request,
			final HttpServletResponse response) throws ServletException,
			IOException {
		final HttpSession session = request.getSession(false);
		
		request.setAttribute("facilities", this.getFacilityHelper()
				.getAllVistAFacilities());
		
		String facilityName = "";
		String facilityStation = "";
		
		Cookie[] vapCookies = request.getCookies();
		gov.va.nvap.svc.facility.data.Facility defaultUserFacility = null;
		if (vapCookies != null){
			for (Cookie vapCookie : vapCookies){
				if (vapCookie.getName().equals("facility")){
					if (NullChecker.isNotEmpty(vapCookie.getValue())) {
						defaultUserFacility = new gov.va.nvap.svc.facility.data.
								Facility(new Long(vapCookie.getValue()));
						defaultUserFacility.setFacilityStation(vapCookie.getValue());
						if(!NullChecker.isNullOrEmpty(defaultUserFacility)){ //cookie is not null
							session.setAttribute("defaultUserFacility", defaultUserFacility);
							facilityStation = defaultUserFacility.getFacilityStation();
							facilityName = this.getFacilityHelper().getFacilityNameByStationId(facilityStation);
							request.setAttribute("visualDefaultUserFacility", facilityName+" ("+facilityStation+")");
							request.setAttribute("facilityState", "manual");
						}
					}
				}
			}
		}
		
		if(NullChecker.isNullOrEmpty(defaultUserFacility)){
			final gov.va.nvap.svc.facility.data.Facility defaultAutoUserFacility = this
					.getFacilityHelper().getFacilityStationByVHAUserId(
							request.getRemoteUser());
			if (!(NullChecker.isNullOrEmpty(defaultAutoUserFacility))) {
				facilityName = defaultAutoUserFacility.getFacilityName();
				facilityStation = defaultAutoUserFacility.getFacilityStation();
				request.setAttribute("visualDefaultUserFacility", facilityName+" ("+facilityStation+")");
				request.setAttribute("facilityState", "auto");
			}else{
				request.setAttribute("facilityState", "none");
			}
		}
		
		this.forward(request, response, "show");
	}
}