/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package gov.va.nvap.web.report;

import gov.va.nvap.common.validation.NullChecker;
import gov.va.nvap.web.dao.PurposeOfUseDAO;
import gov.va.nvap.web.entities.PurposeOfUse;
import gov.va.nvap.web.helper.facility.FacilityHelper;
import gov.va.nvap.web.helper.privacy.ConsentManagementHelper;
import gov.va.nvap.web.util.Paginator;
import gov.va.nvap.web.util.date.DateUtil;
import gov.va.nvap.web.util.Constants;

import java.io.IOException;
import java.util.Calendar;
import java.util.Collection;
import java.util.Date;

import javax.servlet.ServletException;
import javax.servlet.http.Cookie;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpSession;

/**
 * @author David Vazquez
 */
public class DocumentDisclosureReport extends
		gov.va.nvap.web.app.ResponseDispatcherHttpServlet {
	/**
	 *
	 */
	private static final long serialVersionUID = -4913924881974107661L;

	public ConsentManagementHelper getCmsHelper() {
		final ConsentManagementHelper cmsHelper = this.getBean("cmsHelper",
				ConsentManagementHelper.class);
		return cmsHelper;
	}

	/**
	 * Get the facility helper from Spring.
	 */
	public FacilityHelper getFacilityHelper() {
		final FacilityHelper facilityHelper = this.getBean("facilityHelper",
				FacilityHelper.class);
		return facilityHelper;
	}

	public void search(final HttpServletRequest request,
			final HttpServletResponse response) throws ServletException,
			IOException {
		final HttpSession session = request.getSession(false);
		final String startDate = request.getParameter("startDate");
		final String endDate = request.getParameter("endDate");
		final String ssn = request.getParameter("ssn");
		final String lastName = request.getParameter("lastName");
		final String firstName = request.getParameter("firstName");
		final boolean testPatients = "testPatients".equals(request
				.getParameter("testPatients"));
		final String organization = request.getParameter("organization");
                final String organizationName = request.getParameter("organizationName");
		final String facility = request.getParameter("facility");
                final String facilityName = request.getParameter("facilityName");
		final String defaultFacility = request.getParameter("defaultFacility");
		final int recordsPerPage = Integer.parseInt(request
				.getParameter("recordsPerPage"));
                final String source = request.getParameter("source");
		final String testDataOnly = request.getParameter("testDataOnly");

		final String documentDisclosureSortValue = request
				.getParameter("documentDisclosureSortValue");
		final String documentDisclosureSortDirection = request
				.getParameter("documentDisclosureSortDirection");
		 
		final String purposeOfUse = request
				.getParameter("purposeOfUse");
		
		if (this.validate(request, response, "search")) {
			ReportDataProcessor.setStartAndEndDates(session, startDate, endDate, "documentDisclosureQuery");
            
			session.setAttribute("paginator", new Paginator(recordsPerPage));
			session.setAttribute("testPatients", testPatients);
			session.setAttribute("documentDisclosureQuerySSN", ssn);
			session.setAttribute("documentDisclosureQueryLastName", lastName);
			session.setAttribute("documentDisclosureQueryFirstName", firstName);
			session.setAttribute("documentDisclosureQueryOrganization",
					organization);
                        session.setAttribute("documentDisclosureQueryOrgName", organizationName);
			session.setAttribute("purposeOfUse", purposeOfUse);
			session.setAttribute("documentDisclosureQueryFacility", facility);
                        session.setAttribute("documentDisclosureQueryFacilityName", facilityName);

			session.setAttribute("documentDisclosureSortValue",
					documentDisclosureSortValue);
			session.setAttribute("documentDisclosureSortDirection",
					documentDisclosureSortDirection);
                        session.setAttribute("documentDisclosureSource", source);
			// Just to be safe
			session.removeAttribute("documentDisclosureQueryICN");
			if (!NullChecker.isNullOrEmpty(defaultFacility)
					&& defaultFacility.equalsIgnoreCase("ON")) {
				response.addCookie(this.setFacilityCookie(facility));
			}
			
			if (!NullChecker.isNullOrEmpty(testDataOnly)
					&& testDataOnly.equalsIgnoreCase("ON")) {
				session.setAttribute("testDataOnly", true);
				// TODO communicate to adapter to only use test data
			}else{
				session.setAttribute("testDataOnly", false);
			}
			
			this.forward(request, response, "success");
		} else {
			this.forward(request, response, "validate");
		}
	}

	/**
	 * If the URL starts with redirect: then redirect to a different location
	 * with the same parameters.
	 */
	private Cookie setFacilityCookie(final String setFacility) {

		Cookie vapCookie = null;

		if (setFacility != null) {
			if (setFacility.equals("")) {
				vapCookie = new Cookie("facility", "");
				vapCookie.setMaxAge(0);
			} else {
				vapCookie = new Cookie("facility", setFacility);
				vapCookie.setMaxAge(5 * 365 * 24 * 60 * 60);
			}

			// vapCookie.setPath("/");
			// vapCookie.setDomain("localhost:7001/nhin_vap_web");
		}
		return vapCookie;
	}

	@Override
	public void unspecified(final HttpServletRequest request,
			final HttpServletResponse response) throws ServletException,
			IOException {
		final HttpSession session = request.getSession(false);

		session.removeAttribute("documentDisclosureQueryICN");
		session.removeAttribute("documentDisclosureSSN");
		session.removeAttribute("documentDisclosureLastName");
		session.removeAttribute("documentDisclosureFirstName");

		session.removeAttribute("documentDisclosureQueryStartDate");
		session.removeAttribute("documentDisclosureQueryEndDate");

		session.removeAttribute("documentDisclosureQueryFacility");
		session.removeAttribute("documentDisclosureQueryOrganization");
		session.removeAttribute("testPatients");
		session.removeAttribute("paginator");
		session.removeAttribute("documentDisclosureResults");
		session.removeAttribute("testDataOnly");

		// sort
		session.removeAttribute("documentDisclosureSortValue");
		session.removeAttribute("documentDisclosureSortDirection");

		request.setAttribute("facilities", this.getFacilityHelper()
				.getAllVistAFacilities());
		request.setAttribute("organizations", this.getCmsHelper()
				.getAllOrganizations());

		if (Constants.PURPOSE_OF_USE_FLAG){
			final PurposeOfUseDAO purposeOfUseDAO = this.getBean(
					"purposeOfUseDAO", PurposeOfUseDAO.class);
			Collection<PurposeOfUse> pous = purposeOfUseDAO.findPurposeOfUse();
			request.setAttribute("pous", pous);
		}
		request.setAttribute("pouFlag", Constants.PURPOSE_OF_USE_FLAG);
		
		// Get the Default Facility by user id
		/*
		final gov.va.nvap.svc.facility.data.Facility defaultUserFacility = 
			this.getFacilityHelper().getFacilityStationByVHAUserId(request.getRemoteUser());
		if (NullChecker.isNotEmpty(defaultUserFacility)) {
			session.setAttribute("defaultUserFacility",
					defaultUserFacility);
		}
		*/
		
		final gov.va.nvap.svc.facility.data.Facility defaultUserFacility = 
			(gov.va.nvap.svc.facility.data.Facility) session.getAttribute("defaultUserFacility");
		
		this.forward(request, response, "show");
	}

}
