package gov.va.nvap.web.report;

import gov.va.nvap.web.app.ResponseDispatcherHttpServlet;
import gov.va.nvap.web.helper.facility.FacilityHelper;
import gov.va.nvap.web.helper.privacy.ConsentManagementHelper;
import gov.va.nvap.web.util.Paginator;

import java.io.IOException;
import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.Map;

import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpSession;

public class ExpiringConsentReport extends ResponseDispatcherHttpServlet {

	/**
	 * Serial Version UID
	 */
	private static final long serialVersionUID = -1303638679062246256L;

	/**
	 * Gets the consent management helper from Spring.
	 */
	private ConsentManagementHelper getCmsHelper() {
		return getBean("cmsHelper", ConsentManagementHelper.class);
	}
    
	/**
	 * Gets the facility helper from Spring.
	 */
	private FacilityHelper getFacilityHelper() {
		return getBean("facilityHelper", FacilityHelper.class);
	}
    
    private Map<String, String> getPreLoadValues() {
        return getCmsHelper().getExpiringConsentConfiguration();
    }

	// When "Search" is clicked
	public void search(final HttpServletRequest request,
			final HttpServletResponse response) throws ServletException,
			IOException {
		
		// Get values from search fields
		final HttpSession session = request.getSession(false);
		final int recordsPerPage = Integer.parseInt(request.getParameter("recordsPerPage"));
        final String startDate = request.getParameter("startDate");
        final String endDate = request.getParameter("endDate");
        final String facilityName = request.getParameter("facility");
        final String consentTypeName = request.getParameter("consentTypeName");
        final String userId = request.getParameter("enteredBy");
		final String inboundSortValue = request.getParameter("inboundSortValue");
		final String inboundSortDirection = request.getParameter("inboundSortDirection");
        final boolean testPatients = "testPatients".equals(request.getParameter("testPatients"));

		if (this.validate(request, response, "search")) {
            ReportDataProcessor.setStartAndEndDates(session, startDate, endDate, "");
            session.setAttribute("facilityName", facilityName);
            session.setAttribute("consentTypeName", consentTypeName);
            session.setAttribute("userId", userId);
			session.setAttribute("inboundSortValue", inboundSortValue);
			session.setAttribute("inboundSortDirection", inboundSortDirection);
			session.setAttribute("paginator", new Paginator(recordsPerPage));
            session.setAttribute("testPatients", testPatients);
			// Redirect according to web.xml success param value
			this.forward(request, response, "success");
		} else {
			this.forward(request, response, "validate");
		}
	}

	// When redirected to the page before any actions
	@Override
	protected void unspecified(final HttpServletRequest request,
			final HttpServletResponse response) throws ServletException,
			IOException {
        final HttpSession session = request.getSession(false);
        final int dayWindow = Integer.parseInt(this.getPreLoadValues().get("window"));

        session.removeAttribute("startDate");
        session.removeAttribute("endDate");
        session.removeAttribute("facilityName");
        session.removeAttribute("consentTypeName");
        session.removeAttribute("userId");
		session.removeAttribute("paginator");
        session.removeAttribute("testPatients");
        
        DateFormat dateFormat = new SimpleDateFormat("MM/dd/yyyy");
        Calendar calendar = Calendar.getInstance();
        String startDateString = dateFormat.format(calendar.getTime());
        calendar.add(Calendar.DATE, dayWindow);
        String endDateString = dateFormat.format(calendar.getTime());
        session.setAttribute("startDateString", startDateString);
        session.setAttribute("endDateString", endDateString);
        
		request.setAttribute("facilities", this.getFacilityHelper().getAllVistAFacilities());
        
		this.forward(request, response, "show");
	}
}
