/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package gov.va.nvap.web.report;

import gov.va.nvap.common.validation.NullChecker;
import gov.va.nvap.web.app.ResponseDispatcherHttpServlet;
import gov.va.nvap.web.dao.PurposeOfUseDAO;
import gov.va.nvap.web.entities.PurposeOfUse;
import gov.va.nvap.web.helper.facility.FacilityHelper;
import gov.va.nvap.web.helper.privacy.ConsentManagementHelper;
import gov.va.nvap.web.util.Constants;
import gov.va.nvap.web.util.Paginator;
import gov.va.nvap.web.util.date.DateUtil;

import java.io.IOException;
import java.util.Calendar;
import java.util.Collection;
import java.util.Date;

import javax.servlet.ServletException;
import javax.servlet.http.Cookie;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpSession;

/**
 * Documents that are received by the VA from external partners.
 * 
 * @author Asha Amritraj
 */
public class InboundDocumentReport extends ResponseDispatcherHttpServlet {
	/**
	 * Serial UID.
	 */
	private static final long serialVersionUID = 1512959065218653085L;

	public ConsentManagementHelper getCmsHelper() {
		final ConsentManagementHelper cmsHelper = this.getBean("cmsHelper",
				ConsentManagementHelper.class);
		return cmsHelper;
	}

	/**
	 * Get the facility helper from Spring.
	 */
	public FacilityHelper getFacilityHelper() {
		final FacilityHelper facilityHelper = this.getBean("facilityHelper",
				FacilityHelper.class);
		return facilityHelper;
	}

	public void search(final HttpServletRequest request,
			final HttpServletResponse response) throws ServletException,
			IOException {
		final HttpSession session = request.getSession(false);
		final String startDate = request.getParameter("startDate");
		final String endDate = request.getParameter("endDate");
		final String ssn = request.getParameter("ssn");
		final String lastName = request.getParameter("lastName");
		final String firstName = request.getParameter("firstName");
		final String remoteOrg = request.getParameter("remoteOrganization");
                final String remoteOrgName = request.getParameter("organizationName");
		final String facility = request.getParameter("facility");
                final String facilityName = request.getParameter("facilityName");
		final String defaultFacility = request.getParameter("defaultFacility");
		final String inboundSortValue = request
				.getParameter("inboundSortValue");
		final String inboundSortDirection = request
				.getParameter("inboundSortDirection");
		final String testDataOnly = request.getParameter("testDataOnly");
		final boolean testPatients = "testPatients".equals(request
				.getParameter("testPatients"));
		final int recordsPerPage = Integer.parseInt(request
				.getParameter("recordsPerPage"));

		final String purposeOfUse = request
				.getParameter("purposeOfUse");
		
		if (this.validate(request, response, "search")) {
			ReportDataProcessor.setStartAndEndDates(session, startDate, endDate, "inboundDocumentQuery");

			session.setAttribute("paginator", new Paginator(recordsPerPage));
			session.setAttribute("inboundDocumentQuerySSN", ssn);
			session.setAttribute("inboundDocumentQueryLastName", lastName);
			session.setAttribute("inboundDocumentQueryFirstName", firstName);
			session.setAttribute("purposeOfUse", purposeOfUse);
			session.setAttribute("inboundDocumentQueryFacility", facility);
                        session.setAttribute("inboundDocumentQueryFacilityName", facilityName);
			session.setAttribute("inboundDocumentQueryRemoteOrganization",
					remoteOrg);
                        session.setAttribute("inboundDocumentQueryRemoteOrgName", remoteOrgName);
			session.setAttribute("testPatients", testPatients);
			session.setAttribute("inboundSortValue", inboundSortValue);
			session.setAttribute("inboundSortDirection", inboundSortDirection);

			// Just to be safe
			session.removeAttribute("inboundDocumentQueryICN");

			if (!NullChecker.isNullOrEmpty(defaultFacility)
					&& defaultFacility.equalsIgnoreCase("ON")) {
				response.addCookie(this.setFacilityCookie(facility));
			}
			
			if (!NullChecker.isNullOrEmpty(testDataOnly)
					&& testDataOnly.equalsIgnoreCase("ON")) {
				session.setAttribute("testDataOnly", true);
				// TODO communicate to adapter to only use test data
			}else{
				session.setAttribute("testDataOnly", false);
			}
			
			this.forward(request, response, "success");
		} else {
			this.forward(request, response, "validate");
		}
	}

	/**
	 * If the URL starts with redirect: then redirect to a different location
	 * with the same parameters.
	 */
	private Cookie setFacilityCookie(final String setFacility) {

		Cookie vapCookie = null;

		if (setFacility != null) {
			if (setFacility.equals("")) {
				vapCookie = new Cookie("facility", "");
				vapCookie.setMaxAge(0);
			} else {
				vapCookie = new Cookie("facility", setFacility);
				vapCookie.setMaxAge(5 * 365 * 24 * 60 * 60);
			}

			// vapCookie.setPath("/");
			// vapCookie.setDomain("localhost:7001/nhin_vap_web");
		}
		return vapCookie;
	}

	@Override
	protected void unspecified(final HttpServletRequest request,
			final HttpServletResponse response) throws ServletException,
			IOException {
		final HttpSession session = request.getSession(false);

		session.removeAttribute("inboundDocumentQueryICN");
		session.removeAttribute("inboundDocumentQuerySSN");
		session.removeAttribute("inboundDocumentQueryLastName");
		session.removeAttribute("inboundDocumentQueryFirstName");

		session.removeAttribute("inboundDocumentQueryStartDate");
		session.removeAttribute("inboundDocumentQueryEndDate");

		session.removeAttribute("inboundDocumentQueryFacility");
		session.removeAttribute("inboundDocumentQueryRemoteOrganization");
		session.removeAttribute("testPatients");
		session.removeAttribute("paginator");
		session.removeAttribute("inboundDocumentResults");
		session.removeAttribute("testDataOnly");
		
		// sort
		session.removeAttribute("inboundSortValue");
		session.removeAttribute("inboundSortDirection");

		request.setAttribute("facilities", this.getFacilityHelper()
				.getAllVistAFacilities());
		//request.setAttribute("organizations", this.getCmsHelper()
		//		.getAllowedOrganizations());
		request.setAttribute("organizations", this.getCmsHelper()
						.getAllowedNonConsumerOnlyOrganizations());
		
		if (Constants.PURPOSE_OF_USE_FLAG){
			final PurposeOfUseDAO purposeOfUseDAO = this.getBean(
					"purposeOfUseDAO", PurposeOfUseDAO.class);
			Collection<PurposeOfUse> pous = purposeOfUseDAO.findPurposeOfUse();
			request.setAttribute("pous", pous);
		}
		request.setAttribute("pouFlag", Constants.PURPOSE_OF_USE_FLAG);
		
		// Get the Default Facility by user id
		//TODO put this in same place for each place it is used, once for each report
		/*
		final gov.va.nvap.svc.facility.data.Facility defaultUserFacility = 
			this.getFacilityHelper().getFacilityStationByVHAUserId(request.getRemoteUser());
		if (NullChecker.isNotEmpty(defaultUserFacility)) {
			session.setAttribute("defaultUserFacility",
					defaultUserFacility);
		}
		*/

		final gov.va.nvap.svc.facility.data.Facility defaultUserFacility = 
			(gov.va.nvap.svc.facility.data.Facility) session.getAttribute("defaultUserFacility");
		
		this.forward(request, response, "show");
	}

}
