package gov.va.nvap.web.report;

import gov.va.nvap.common.transformer.TransformerException;
import gov.va.nvap.common.validation.NullChecker;
import gov.va.nvap.common.xpath.XPathException;
import gov.va.nvap.service.audit.ConsentAuditType;
import gov.va.nvap.web.app.ResponseDispatcherHttpServlet;
import gov.va.nvap.web.consent.audit.AuditedConsentEx;
import gov.va.nvap.web.consent.audit.dao.AuditedConsentDAO;
import gov.va.nvap.web.helper.document.DocumentHelper;
import gov.va.nvap.web.helper.privacy.ConsentManagementHelper;
import gov.va.nvap.web.helper.document.MediaType;
import gov.va.nvap.web.helper.document.RepresentationType;
import gov.va.nvap.web.helper.facility.FacilityHelper;
import gov.va.nvap.web.helper.report.ReportHelper;
import gov.va.nvap.web.util.Constants;
import gov.va.nvap.web.util.FieldChecks;
import gov.va.nvap.web.util.Paginator;
import gov.va.nvap.web.util.xls.ExcelExporter;
import java.io.ByteArrayInputStream;

import java.io.IOException;
import java.io.OutputStream;
import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.LinkedHashMap;

import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpSession;
import org.apache.poi.ss.usermodel.Workbook; 

/**
 * @author             
 */
public class OptInOptOutReportResults extends ResponseDispatcherHttpServlet {

	/**
	 * Serial UID.
	 */
	private static final long serialVersionUID = -1131510956121479062L;

        /**
              * Get the Consent Management Service Helper class from Spring. The
          * Helper class can be used to access consent management functions on
          * the server side.
        * 
        * @return helper class for consent management purposes
        */
        public ConsentManagementHelper getCmsHelper() {
               final ConsentManagementHelper cmsHelper = this.getBean("cmsHelper",
                               ConsentManagementHelper.class);
               return cmsHelper;
        }

	/**
	 * Export to excel - This method is called when the user clicks on the excel
	 * icon in the JSP. Reflection is used in the ResponseDispatcherHttpServlet
	 * to call this method based on the http parameters.
	 * 
	 * @param request
	 *            HTTP Request
	 * @param response
	 *            HTTP Response
	 * @throws ServletException
	 * @throws IOException
	 */
	/*
	 * uncomment this method and excel icon in corresponding jsp to re-enable excel reporting
	 *
         */
	 public void exportToExcel(final HttpServletRequest request, 
             final HttpServletResponse response) throws ServletException, IOException { 
            //Get the session 
            final HttpSession session = request.getSession(false); //
            //Get the results from the adapter with the paginator set to -1 (all //  results) 
            final List<Map<String, Object>> results = this.getResults(
            request.getSession(), new Paginator(this.getReportHelper().getConsentDirectivesReportCapAsInt()));
            
            // Create a map of key which is based on the result data key and the heading value 
            //The heading is used to create the column headers and the
            //key is used to pull the data from the results 
            final Map<String,String> reportMap = new LinkedHashMap<String, String>();
            
            // Set the heading parameters 
            final Map<String, List<Object>> headingParameters = new LinkedHashMap<String, List<Object>>();
            
            final Object startDateStr = session.getAttribute("optInOptOutQueryStartDate"); 
            final Object endDateStr = session .getAttribute("optInOptOutQueryEndDate"); 

            LinkedHashMap<String,Object> filters = new LinkedHashMap<String,Object>();
            //List all possible filters.
            filters.put("SSN", session.getAttribute("optInOptOutQuerySsn"));
            filters.put("Last Name", session.getAttribute("optInOptOutQueryLastName"));
            filters.put("First Name", session.getAttribute("optInOptOutQueryFirstName"));
            filters.put("Start Date", this.getReportHelper().getFormattedDate((Date)startDateStr));
            filters.put("End Date", this.getReportHelper().getFormattedDate((Date)endDateStr));
            filters.put("VA Facility", session.getAttribute("optInOptOutQueryFacilityName"));
            filters.put("Consent Type", session.getAttribute("optInOptOutQueryConsentTypeName"));
            filters.put("Inactivation Reason", session.getAttribute("inactivationReason"));
            filters.put("Entered By", session.getAttribute("enteredBy"));
            filters.put("VISN", session.getAttribute("optInOptOutQueryVisnName"));
            //Iterate over all potential filters and add non-default ones to
            //the heading parameters.
            for (Map.Entry<String,Object> filter : filters.entrySet()){
                String filterName = filter.getKey();
                Object value = filter.getValue();
                List<Object> headingValue = new ArrayList<Object>();
                if (!(value == null) && !("".equals(value)) && !("Unknown".equals(value))
                        && !("ALL".equals(value))){
                    headingValue.add(value);
                    headingParameters.put(filterName, headingValue);
                }
            }
            
            if ((Boolean)session.getAttribute("testPatients")){
                List<Object> headingValue = new ArrayList<Object>();
                headingValue.add("Yes");
                headingParameters.put("Include test patients", headingValue);
            }
            
            if (NullChecker.isNullOrEmpty(results)){
                reportMap.put("", "No records were found.");
            }
            
            else {
                reportMap.put("ssnMasked", "SSN"); 
                reportMap.put("lastName", "Patient Last Name"); 
                reportMap.put("firstName", "Patient First Name");
                reportMap.put("auditTimeFormatted", "Time of Event");
                reportMap.put("patientDateFormatted", "Patient Signature/Patient Deceased Date"); 
                reportMap.put("purposeOfUse", "Purpose of Use"); 
                reportMap.put("consentType", "Consent Type");
                reportMap.put("inactivationReason", "Inactivation Reason");
                reportMap.put("userId", "Entered By"); 
                reportMap.put("facilityName","VA Facility");
                reportMap.put("visnName","VISN");
            }
            
            // Create workbook 
            final Workbook wb = this.getExcelExporter().exportToExcel(
            "Consent Directive Detailed Report", "Consent Directive Detailed Report",
            reportMap, results, headingParameters, null);
            
            
            
            // Write Excel to Stream
            this.getExcelExporter().writeExcelToStream(wb, response); 
        }
	
	public DocumentHelper getDocumentHelper() {
		return this.getBean("adapterDocumentHelper", DocumentHelper.class);
	}

	/**
	 * Get the excel exporter class from Spring.
	 * 
	 * @return the excel exporter object
	 */
	public ExcelExporter getExcelExporter() {
		final ExcelExporter excelExporter = this.getBean("excelExporter",
				ExcelExporter.class);
		return excelExporter;
	}

	/**
	 * Get the facility helper from Spring.
	 */
	public FacilityHelper getFacilityHelper() {
		return this.getBean("facilityHelper", FacilityHelper.class);
	}

	public ReportHelper getReportHelper() {
		final ReportHelper reportHelper = this.getBean("reportHelper",
				ReportHelper.class);
		return reportHelper;
	}

	private List<Map<String, Object>> getResults(final HttpSession session,
			final Paginator paginator) throws ServletException {
		final String firstName = (String) session
				.getAttribute("optInOptOutQueryFirstName");
		final String lastName = (String) session
				.getAttribute("optInOptOutQueryLastName");
		final String ssn = (String) session.getAttribute("optInOptOutQuerySsn");
		final String facilityName = (String) session
				.getAttribute("optInOptOutQueryFacilityName");
		final String visnName = (String) session
				.getAttribute("optInOptOutQueryVisnName");
		final String consentType = (String) session
				.getAttribute("optInOptOutQueryConsentType");
                final String consentTypeName = (String) session
                                .getAttribute("optInOptOutQueryConsentTypeName");
		final String inactivationReason = (String) session
				.getAttribute("inactivationReason");
		final Date sd = (Date) session
				.getAttribute("optInOptOutQueryStartDate");
		final Date ed = (Date) session.getAttribute("optInOptOutQueryEndDate");
		final Boolean includeTestPatients = (Boolean) session
				.getAttribute("testPatients");
		final String enteredBy = (String) session.getAttribute("enteredBy");

		final AuditedConsentDAO auditedConsentDAO = this.getBean(
				"auditedConsentDAO", AuditedConsentDAO.class);
		List<Object> auditedConsentExList = null;

		final String optInOptOutSortValue = (String) session
				.getAttribute("optInOptOutSortValue");
		final String optInOptOutSortDirection = (String) session
				.getAttribute("optInOptOutSortDirection");
                
                /*
                // First, do a query without any pagination so we can get a total row count.
                // This is for the "Download Patient Notification Letters" feature
                int totalRows = auditedConsentDAO.getEvents(sd, ed, ssn,
				firstName, lastName, facilityName, visnName, consentType,
				inactivationReason, enteredBy, 0,
				0, optInOptOutSortValue,
				optInOptOutSortDirection, includeTestPatients).size();

                session.setAttribute("totalRows", totalRows);
                session.setAttribute("consentDirectiveLettersMaxTime", Constants.getConsentDirectiveLettersMaxTime());
                session.setAttribute("consentDirectivePerLetterTime", Constants.getConsentDirectivePerLetterTime());
                */
        
                // The result of the query is a list of arrays.
                // Each array is a [AuditedConsent, Facility, Visn]
                auditedConsentExList = auditedConsentDAO.getEvents(sd, ed, ssn,
				firstName, lastName, facilityName, visnName, consentType,
				inactivationReason, enteredBy, paginator.getCurrentPage(),
				paginator.getRecordsPerPage(), optInOptOutSortValue,
				optInOptOutSortDirection, includeTestPatients);
                
		final List<Map<String, Object>> results = new ArrayList<Map<String, Object>>();
                if (NullChecker.isNotEmpty(auditedConsentExList)) {
                        for (final Object o : auditedConsentExList) {
                                AuditedConsentEx auditType = (AuditedConsentEx)o;
                                
				final Map<String, Object> resultMap = new HashMap<String, Object>();

				// Replace SSN with Mask
                ReportDataProcessor.addSsnToResultMap(resultMap, auditType.getPatientSsn());
                
				resultMap.put("icn", auditType.getPatientId());
				resultMap.put("auditTime", auditType.getTimeOfEvent());
				resultMap.put("auditTimeFormatted", this.getReportHelper()
						.getFormattedDateTime(auditType.getTimeOfEvent()));
				resultMap.put("userId", auditType.getUserId());
				resultMap.put("actionType", auditType.getActionType());

				String consentTypeRow = auditType.getConsentType();
                if (consentTypeRow.contains("NwHIN")) {
                    consentTypeRow = consentTypeRow.replace("NwHIN", Constants.getOrganizationName());
                }
				resultMap.put("consentType", consentTypeRow);
				resultMap.put("firstName", auditType.getPatientGivenName());
				resultMap.put("lastName", auditType.getPatientLastName());
				resultMap.put("purposeOfUse", auditType.getPouValue());

				String inactivationReasonRow = auditType.getOptoutReason();
				resultMap.put("inactivationReason", inactivationReasonRow);
				resultMap.put("patientDate", auditType.getCreatedDate());
				resultMap.put("patientDateFormatted", this.getReportHelper()
						.getFormattedDate(auditType.getCreatedDate()));

				// final String vaFacility = this.getFacilityHelper()
				// .getFacilityNameByStationId(auditType.getFacility());
				// resultMap.put("facility",
				// ReportDataProcessor.fixStation(vaFacility));
				resultMap.put("facilityName", ReportDataProcessor
						.fixStation(auditType.getFacilityName()));
                                
                                resultMap.put("visnName", auditType.getVisnName());
                                
				if (ConsentAuditType.OPTIN_AUDIT_TYPE.value().equals(
						auditType.getActionType())
						|| ConsentAuditType.OPTOUT_AUDIT_TYPE.value().equals(
								auditType.getActionType())) {
					resultMap.put("auditId", auditType.getConsentAuditId());
					// Get the CDA R2 Content from Audit
					final String content = auditedConsentDAO
							.getMessageContent(auditType.getConsentAuditId());
					if (NullChecker.isNotEmpty(content)) {
						try {
							// Get the attachment inside the audit
							final String attachment = this.getDocumentHelper()
									.getPrivacyConsentDirectiveAttachment(
											content);
							// Set the flag
							if (NullChecker.isNotEmpty(attachment)) {
								resultMap.put("hasAttachment", true);
							} else {
								resultMap.put("hasAttachment", false);
							}
						} catch (final TransformerException ex) {
							throw new ServletException(ex);
						} catch (final XPathException ex) {
							throw new ServletException(ex);
						}
					}
				}
				results.add(FieldChecks.replaceEmptyOrNullWithSpace(resultMap));
			}
		}

		// Check if any of the patient match criteria exists
		if (NullChecker.isNotEmpty(ssn) || NullChecker.isNotEmpty(lastName)
				|| NullChecker.isNotEmpty(firstName)) {
			// Default to unknown
			String resultIcn = "";
                        if (NullChecker.isNotEmpty(auditedConsentExList)) {
                                final AuditedConsentEx result = (AuditedConsentEx)auditedConsentExList.get(0);
				resultIcn = result.getPatientId();

				if (NullChecker.isNotEmpty(resultIcn)) {
                                        for (final Object o : auditedConsentExList) {
                                                AuditedConsentEx consent = (AuditedConsentEx)o;
						if (!resultIcn.equals(consent.getPatientId())) {
							resultIcn = "Multiple ICNs";
							break;
						}
					}
				} else if (NullChecker.isNotEmpty(result.getPatientGivenName())
						&& NullChecker.isNotEmpty(result.getPatientLastName())
						&& NullChecker.isNotEmpty(result.getPatientSsn())) {
					final String patientGivenName = result
							.getPatientGivenName();
					final String patientLastName = result.getPatientLastName();
					final String patientSsn = result.getPatientSsn();
					boolean multipleIcns = false;
                                        for (final Object o : auditedConsentExList) {
                                                AuditedConsentEx consent = (AuditedConsentEx)o;
						if (!patientGivenName.equals(consent
								.getPatientGivenName())
								|| !patientLastName.equals(consent
										.getPatientLastName())
								|| !patientSsn.equals(consent.getPatientSsn())) {
							multipleIcns = true;
							break;
						}
					}
					if (multipleIcns) {
						resultIcn = "Multiple ICNs";
					} else {
						resultIcn = this.getReportHelper().resolveICN(
								result.getPatientGivenName(),
								result.getPatientLastName(),
								result.getPatientSsn());
					}
				}
			} else if (NullChecker.isNotEmpty(ssn)
					&& NullChecker.isNotEmpty(lastName)
					&& NullChecker.isNotEmpty(firstName)) {
				// Try to get from what was typed
				resultIcn = this.getReportHelper().resolveICN(firstName,
						lastName, ssn);
			}
			// Set in Session
			if (NullChecker.isEmpty(resultIcn)) {
				resultIcn = "Unknown";
			}
			session.setAttribute("optInOptOutQueryICN", resultIcn);
		}

		return results;
	}

	private boolean isPaginatorPresent(final HttpSession session) {
		return !NullChecker.isNullOrEmpty(session.getAttribute("paginator"));
	}

	public void next(final HttpServletRequest request,
			final HttpServletResponse response) throws ServletException,
			IOException {
		final HttpSession session = request.getSession(false);
		if (this.isPaginatorPresent(session)) {
			final Paginator paginator = (Paginator) session
					.getAttribute("paginator");
			paginator.next(request);
			session.setAttribute("paginator", paginator);
			session.setAttribute("optInOptOutResults",
					this.getResults(session, paginator));
			this.forward(request, response, "show");
		} else {
			this.forward(request, response, "noshow");
		}
	}

	public void prev(final HttpServletRequest request,
			final HttpServletResponse response) throws ServletException,
			IOException {
		final HttpSession session = request.getSession(false);
		if (this.isPaginatorPresent(session)) {
			final Paginator paginator = (Paginator) session
					.getAttribute("paginator");
			paginator.previous(request);
			session.setAttribute("paginator", paginator);
			session.setAttribute("optInOptOutResults",
					this.getResults(session, paginator));
			this.forward(request, response, "show");
		} else {
			this.forward(request, response, "noshow");
		}
	}

	@Override
	public void unspecified(final HttpServletRequest request,
			final HttpServletResponse response) throws ServletException,
			IOException {
		final HttpSession session = request.getSession(false);

		// sort
		final String optInOptOutSortValue = request
				.getParameter("optInOptOutSortValue");
		final String optInOptOutSortDirection = request
				.getParameter("optInOptOutSortDirection");
		session.setAttribute("optInOptOutSortValue", optInOptOutSortValue);
		session.setAttribute("optInOptOutSortDirection",
				optInOptOutSortDirection);

		if (this.isPaginatorPresent(session)) {
			Paginator paginator = (Paginator) session.getAttribute("paginator");
			if (NullChecker.isEmpty(paginator)) {
				paginator = new Paginator(0);
				session.setAttribute("paginator", paginator);
			}
			paginator.reset();
			session.setAttribute("optInOptOutResults",
					this.getResults(session, paginator));
			this.forward(request, response, "show");
		} else {
			this.forward(request, response, "noshow");
		}
	}

	public void view(final HttpServletRequest request,
			final HttpServletResponse response) throws ServletException,
			IOException {
		final HttpSession session = request.getSession(false);
		final String documentUniqueId = request
				.getParameter("documentUniqueId");

		final AuditedConsentDAO auditedConsentDAO = this.getBean(
				"auditedConsentDAO", AuditedConsentDAO.class);
		final String document = auditedConsentDAO.getMessageContent(Long
				.parseLong(documentUniqueId));

		session.setAttribute("document", document);
		session.setAttribute("documentUniqueId", documentUniqueId);

		try {
			final String attachment = this.getDocumentHelper()
					.getPrivacyConsentDirectiveAttachment(document);
			if (NullChecker.isNotEmpty(attachment)) {
				this.getDocumentHelper()
						.writeAttachmentToStream(
								response,
								attachment,
								MediaType.fromValue(this.getDocumentHelper()
										.getPrivacyConsentDirectiveMediaType(
												document)),
								RepresentationType.B64);
			} else {
				session.setAttribute("informationMessage",
						"There are no attachments associated with this consent directive.");
				this.forward(request, response, "noattachment");
			}
		} catch (final TransformerException ex) {
			throw new ServletException(ex);
		} catch (final XPathException ex) {
			throw new ServletException(ex);
		}
	}
    
        public void download(final HttpServletRequest request,
                            final HttpServletResponse response) throws ServletException,
                            IOException {

            System.out.println("OptInOptOutReportResults.java::download()");
            //this.forward(request, response, "noshow");
            
            final HttpSession session = request.getSession(false);
            final String firstName = (String) session
                            .getAttribute("optInOptOutQueryFirstName");
            final String lastName = (String) session
                            .getAttribute("optInOptOutQueryLastName");
            final String ssn = (String) session.getAttribute("optInOptOutQuerySsn");
            final String facilityName = (String) session
                            .getAttribute("optInOptOutQueryFacilityName");
            final String visnName = (String) session
                            .getAttribute("optInOptOutQueryVisnName");
            final String consentType = (String) session
                            .getAttribute("optInOptOutQueryConsentType");
            final String inactivationReason = (String) session
                            .getAttribute("inactivationReason");
            Date sd = (Date) session
                            .getAttribute("optInOptOutQueryStartDate");
            if (sd == null) {
                sd = new Date(0L);
            }
            Date ed = (Date) session.getAttribute("optInOptOutQueryEndDate");
            if (ed == null) {
                ed = new Date();
            }
            final Boolean includeTestPatients = (Boolean) session
                            .getAttribute("testPatients");
            final String enteredBy = (String) session.getAttribute("enteredBy");

            final ByteArrayInputStream byteInputStream = 
                    this.getCmsHelper().getConsentDirectiveReportLetters(firstName, lastName, ssn, facilityName,
                        visnName, consentType, inactivationReason, sd, ed, includeTestPatients, enteredBy);

            response.setContentType("application/pdf");
            response.addHeader("Content-Disposition", "attachment; filename=patient_letters.pdf");

            OutputStream responseOutStream = response.getOutputStream();

            byte[] buffer = new byte[4096];
            int bytesRead;
            int totalBytes = 0;

            while ((bytesRead = byteInputStream.read(buffer)) != -1) {
                responseOutStream.write(buffer, 0, bytesRead);
                totalBytes += bytesRead;
            }

            response.setContentLength(totalBytes);

            byteInputStream.close();
            responseOutStream.flush();

            this.forward(request, response, "success");
        }

}
