package gov.va.nvap.web.report;

import gov.va.med.nhin.adapter.audit.PageInfoType;
import gov.va.nvap.privacy.ConsentDirectiveExpirationRequest;
import gov.va.nvap.privacy.ConsentDirectiveExpirationResponse;
import gov.va.med.nhin.adapter.audit.SortDirection;
import gov.va.med.nhin.adapter.audit.SortFieldType;
import gov.va.med.nhin.adapter.audit.SortFieldsType;
import gov.va.nvap.common.validation.NullChecker;
import gov.va.nvap.privacy.ConsentDirectiveDetailedExpirationRequest;
import gov.va.nvap.privacy.ConsentDirectiveDetailedExpirationResponse;
import gov.va.nvap.privacy.ConsentDirectiveReferenceType;
import gov.va.nvap.privacy.ConsentType;
import gov.va.nvap.privacy.FacilityOptInConsentType;
import gov.va.nvap.svc.facility.data.Facility;
import gov.va.nvap.web.app.ResponseDispatcherHttpServlet;
import gov.va.nvap.web.helper.facility.FacilityHelper;
import gov.va.nvap.web.helper.privacy.ConsentManagementHelper;
import gov.va.nvap.web.helper.report.ReportHelper;
import gov.va.nvap.web.util.Constants;
import gov.va.nvap.web.util.Paginator;
import gov.va.nvap.web.util.xls.ExcelExporter;

import java.io.IOException;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collection;
import java.util.HashMap;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;

import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpSession;
import org.apache.poi.ss.usermodel.Workbook;

/**
 * @since 12/04/2014
 * @author                  
 */
public class OptInSummaryReport extends ResponseDispatcherHttpServlet {

    /**
	 * 
	 */
	private static final long serialVersionUID = -3952503136988071993L;
    
    /**
    * Get the excel exporter class from Spring.
    * 
    * @return the excel exporter object
    */
   public ExcelExporter getExcelExporter() {
    final ExcelExporter excelExporter = this.getBean("excelExporter",
                    ExcelExporter.class);
    return excelExporter;
   }    
   
   public void exportToExcel(final HttpServletRequest request, 
        final HttpServletResponse response) throws ServletException, IOException { 
        //Get the session 
        final HttpSession session = request.getSession(false); //
        //Get the results from the adapter with the paginator set to -1 (all //  results) 
        final List<List<String>> results = this.getResults(request.getSession());
        // Create a map of key which is based on the result data key and the heading value 
        //The heading is used to create the column headers and the
        //key is used to pull the data from the results 
        final Map<String,String> reportMap = new LinkedHashMap<String, String>();
        reportMap.put("Facility", "Facility");
        reportMap.put("eHealth Exchange Authorization", "eHealth Exchange Authorization");
        reportMap.put("SSA Authorization", "SSA Authorization");
        reportMap.put("Facility Total", "Total");

        //Remap the results to List<Map<String,Object>> so it can be passed to export class.
        List<Map<String,Object>> resultsMap = new ArrayList<Map<String,Object>>();
        List<String> mapKeys = results.get(0);
        results.remove(0);
        for (List<String> listEntry: results){
            Map<String,Object> mapEntry = new LinkedHashMap<String,Object>();
            mapEntry.put(mapKeys.get(0),listEntry.get(0));
            mapEntry.put(mapKeys.get(1),listEntry.get(1));
            mapEntry.put(mapKeys.get(2),listEntry.get(2));
            mapEntry.put(mapKeys.get(3),listEntry.get(3));
            resultsMap.add(mapEntry);
        }
        
        // Set the heading parameters 
        final Map<String, List<Object>> headingParameters = new LinkedHashMap<String, List<Object>>();
        
        if ((Boolean)session.getAttribute("testPatients")){
            List<Object> headingValue = new ArrayList<Object>();
            headingValue.add("Yes");
            headingParameters.put("Include test patients", headingValue);
        }
        
         // Create workbook 
        final Workbook wb = this.getExcelExporter().exportToExcel(
        "Opt-In Patients Summary Report", "Opt-In Patients Summary Report",
        reportMap, resultsMap, headingParameters, null);

        // Write Excel to Stream
        this.getExcelExporter().writeExcelToStream(wb, response);
   }
    public ReportHelper getReportHelper() {
        final ReportHelper reportHelper = this.getBean("reportHelper",
                        ReportHelper.class);
        return reportHelper;
    }

	/**
     * Generate a summary report of opted-in patients. Report is a string matrix with a header row, body, and a 
     * footer row. The first column contains facility names, and the last column contains total for each facility.
     * Footer row contains totals for each authorization type and a grand total in the last column. The following
     * diagram shows the layout of the report:
     * 
     *       Facilities  Auth1   Auth2   ...   AuthN   Facility
     *                                                  Total
     * 
     *       Facility1   [###]   [###]   ...   [###]    [###]
     *       Facility2   [###]   [###]   ...   [###]    [###]
     *       ...          ...     ...    ...    ...      ...
     *       FacilityM   [###]   [###]   ...   [###]    [###]
     * 
     *       Total       [###]   [###]   ...   [###]    [###]
     * 
     * @return List<List<String>> Summary report of opted-in patients.
     */
    private List<List<String>> getResults(HttpSession session) {
        
        final Boolean includeTestPatients = (Boolean) session.getAttribute("testPatients");

        List<List<String>> results = new ArrayList<List<String>>();

        try {
        	final ConsentDirectiveDetailedExpirationRequest cdExpirationReq = new ConsentDirectiveDetailedExpirationRequest();
    		
    		cdExpirationReq.setDayRange(-1);
            
            PageInfoType pageInfo = new PageInfoType();
            pageInfo.setPageNumber(0);
            pageInfo.setPageSize(-1);
            cdExpirationReq.setPageInfo(pageInfo);
    		
            ConsentDirectiveDetailedExpirationResponse cmsResponse = this.getBean("cmsHelper", ConsentManagementHelper.class).getAuthorizedConsentForSummary(cdExpirationReq);

            List<FacilityOptInConsentType> consentReferences = null;
            if (cmsResponse != null) {
                consentReferences = cmsResponse.getFacilityOptInConsentReference();
            }

            if ((consentReferences != null) && (consentReferences.size() > 0)) {

                Map<String, Map<String, Integer>> map = new HashMap<String, Map<String, Integer>>();
                Map<String, Integer> counts;
                Collection<Facility> facilities = this.getBean("facilityHelper", FacilityHelper.class).getAllVistAFacilities();
                List<ConsentType> allowedConsentTypes = Arrays.asList(ConsentType.NW_HIN_AUTHORIZATION, ConsentType.SSA_AUTHORIZATION);

                // Step 1: Construct a hashmap of authorization counts per facility.
                for (int i = 0; i < consentReferences.size(); i++) {
                    if (includeTestPatients != null
                            && includeTestPatients == false) {
                        if (this.getReportHelper().isTestPatient(
                                consentReferences.get(i).getPatientSsn())) {
                            continue;
                        }
                    }
                    
                    ConsentType optinConsentType = consentReferences.get(i).getOptinConsentType();
                    if (allowedConsentTypes.contains(optinConsentType)) {
                        String facilityName = consentReferences.get(i).getFacilityName();
                        if (map.containsKey(facilityName)) {
                            counts = map.get(facilityName);
                        } else {
                            counts = new HashMap<String, Integer>();
                            map.put(facilityName, counts);
                            for (ConsentType allowedConsentType : allowedConsentTypes) {
                                counts.put(allowedConsentType.value(), 0);
                            }
                        }
                        counts.put(optinConsentType.value(), counts.get(optinConsentType.value()) + 1);
                    }
                }

                // Step 2: Construct a report based on the hashmap from Step 1.
                Integer grandTotal = 0;
                Map<String, Integer> totalsByConsentType = new HashMap<String, Integer>();
                List<String> row;

                for (ConsentType allowedConsentType : allowedConsentTypes) {
                    totalsByConsentType.put(allowedConsentType.value(), 0);
                }

                // Construct report header
                row = new ArrayList<String>();
                row.add("Facility");
                for (ConsentType allowedConsentType : allowedConsentTypes) {
                    String consentTypeStr = allowedConsentType.value();
                    if (consentTypeStr.contains("NwHIN")) {
                        consentTypeStr = consentTypeStr.replace("NwHIN", Constants.getOrganizationName());
                    }
                    row.add(consentTypeStr);
                }
                row.add("Facility Total");
                results.add(row);

                // Construct report body
                for (Facility facility : facilities) {
                    if (map.containsKey(facility.getFacilityName())) {
                        row = new ArrayList<String>();
                        Integer totalByFacility = 0;
                        counts = map.get(facility.getFacilityName());
                        row.add(facility.getFacilityName());
                        for (ConsentType allowedConsentType : allowedConsentTypes) {
                            Integer count = counts.get(allowedConsentType.value());
                            row.add(count.toString());
                            totalByFacility += count;
                            totalsByConsentType.put(allowedConsentType.value(), totalsByConsentType.get(allowedConsentType.value()) + count);
                        }
                        row.add(totalByFacility.toString());
                        results.add(row);
                        grandTotal += totalByFacility;
                    }
                }

                // Construct report footer
                row = new ArrayList<String>();
                row.add("Total");
                for (ConsentType allowedConsentType : allowedConsentTypes) {
                    row.add(totalsByConsentType.get(allowedConsentType.value()).toString());
                }
                row.add(grandTotal.toString());
                results.add(row);
            }
        } catch (final Exception ex) {
            throw new RuntimeException(ex);
        }
        return results;
    }

    @Override
    protected void unspecified(HttpServletRequest request, HttpServletResponse response) throws ServletException, IOException {
        final Boolean testPatients = "testPatients".equals(request.getParameter("testPatients"));
        final HttpSession session = request.getSession(false);
        session.setAttribute("testPatients", testPatients);
        session.setAttribute("optInSummaryReportResults", getResults(session));
        this.forward(request, response, "show");
    }
}
