package gov.va.nvap.web.report;

import gov.va.nvap.common.validation.NullChecker;
import gov.va.nvap.web.app.ResponseDispatcherHttpServlet;
import gov.va.nvap.web.helper.facility.FacilityHelper;
import gov.va.nvap.web.helper.privacy.ConsentManagementHelper;

import java.io.IOException;

import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpSession;

/**
 * This class handles the search page for Patient Discovery Summary report.
 * 
 * @author                  
 */
public class PatientDiscoverySummaryReport extends ResponseDispatcherHttpServlet{
	/**
	 * Serial UID.
	 */
	private static final long serialVersionUID = 1L;

    /**
     * Get the consent management helper from Spring.
     * @return ConsentManagementHelper Consent management helper
     */
	public ConsentManagementHelper getCmsHelper() {
		return this.getBean("cmsHelper", ConsentManagementHelper.class);
	}

    /**
     * Get the facility helper from Spring.
     * @return FacilityHelper helper
     */
	public FacilityHelper getFacilityHelper() {
		return this.getBean("facilityHelper", FacilityHelper.class);
	}

	public void search(final HttpServletRequest request,
			final HttpServletResponse response) throws ServletException,
			IOException {
		final HttpSession session = request.getSession(false);
		final String startDate = request.getParameter("startDate");
		final String endDate = request.getParameter("endDate");
		final String organization = request.getParameter("organization");
                final String organizationName = request.getParameter("organizationName");
		final String patientFacility = request.getParameter("facility");
                final String patientFacilityName = request.getParameter("facilityName");
	
		if (this.validate(request, response, "search")) {			
			ReportDataProcessor.setStartAndEndDates(session, startDate, endDate, "patientDiscoverySummaryQuery");
			session.setAttribute("patientDiscoverySummaryQueryOrganization", organization);
                        session.setAttribute("patientDiscoverySummaryQueryOrgName", organizationName);
			session.setAttribute("facility", patientFacility);
                        session.setAttribute("facilityName", patientFacilityName);
			this.forward(request, response, "success");
		} else {
			this.forward(request, response, "validate");
		}
	}

	@Override
	protected void unspecified(final HttpServletRequest request,
			final HttpServletResponse response) throws ServletException,
			IOException {
		final HttpSession session = request.getSession(false);
		session.removeAttribute("patientDiscoverySummaryQueryStartDate");
		session.removeAttribute("patientDiscoverySummaryQueryEndDate");
		session.removeAttribute("patientDiscoverySummaryQueryOrganization");
		session.removeAttribute("facility");

		request.setAttribute("facilities", this.getFacilityHelper().getAllVistAFacilities());
		request.setAttribute("organizations", this.getCmsHelper().getAllOrganizations());
		
		this.forward(request, response, "show");
	}    
}
