package gov.va.nvap.web.report;

import gov.va.med.nhin.adapter.audit.ActionType;
import gov.va.med.nhin.adapter.audit.ActionValuesType;
import gov.va.med.nhin.adapter.audit.AuditSummariesType;
import gov.va.med.nhin.adapter.audit.AuditSummaryType;
import gov.va.med.nhin.adapter.audit.FieldType;
import gov.va.med.nhin.adapter.audit.GetAuditsSummary;
import gov.va.med.nhin.adapter.audit.GetAuditsSummaryResponse;
import gov.va.med.nhin.adapter.audit.GroupByFieldsType;
import gov.va.med.nhin.adapter.audit.StringValuesType;
import gov.va.med.nhin.adapter.audit.SummaryFieldsType;
import gov.va.nvap.common.date.GregorianDateUtil;
import gov.va.nvap.common.validation.NullChecker;
import gov.va.nvap.privacy.OrganizationType;
import gov.va.nvap.service.adapter.audit.AdapterAuditManager;
import gov.va.nvap.service.audit.AuditException;
import gov.va.nvap.web.app.ResponseDispatcherHttpServlet;
import gov.va.nvap.web.helper.facility.FacilityHelper;
import gov.va.nvap.web.helper.privacy.ConsentManagementHelper;
import gov.va.nvap.web.helper.report.ReportHelper;
import gov.va.nvap.web.util.date.DateUtil;
import gov.va.nvap.web.util.xls.ExcelExporter;

import java.io.IOException;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Date;
import java.util.HashMap;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;

import javax.ejb.EJB;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpSession;
import org.apache.poi.ss.usermodel.Workbook;

/**
 * This class handles the results page for Patient Discovery Summary report.
 * 
 * @author                  
 */
public class PatientDiscoverySummaryReportResults extends ResponseDispatcherHttpServlet {
    private static final long serialVersionUID = 1L;
    private AdapterAuditManager adapterAuditManager;

    @EJB(beanInterface = AdapterAuditManager.class, mappedName = "AdapterAuditManager")
    public void setAdapterAuditManager(final AdapterAuditManager adapterAuditManager) {
        this.adapterAuditManager = adapterAuditManager;
    }
    
    public void exportToExcel(final HttpServletRequest request, final
      HttpServletResponse response) throws ServletException, IOException { 
        //Get the session final 
        HttpSession session = request.getSession(false);
        //Get the results from the adapter with the paginator set to -1 (all 
        //results) 
        final List<Map<String, Object>> results = (List<Map<String, Object>>)session.getAttribute("patientDiscoverySummaryResults"); 
    //      Create a map of key which is based on the result data key and the 
    //      heading value The heading is used to create the column headers and the
    //      key is used to pull the data from the results 
        final Map<String,String> patientDiscoveryMap = new LinkedHashMap<String, String>();
        patientDiscoveryMap.put("partnerOrg", "eHealth Exchange Organization");
        patientDiscoveryMap.put("vaFacility", "VA Facility");
        patientDiscoveryMap.put("auditsCount", "Audits");
        patientDiscoveryMap.put("countUniqueRealPatients", "Unique Real Patients");
        patientDiscoveryMap.put("matchesFoundRealPatients", "Matches Found for Real Patients");
        patientDiscoveryMap.put("matchFailsRealPatients", "Match Fails for Real Patients");
        patientDiscoveryMap.put("countUniqueTestPatients", "Unique Test Patients");
        patientDiscoveryMap.put("matchesFoundTestPatients", "Matches Found for Test Patients");
        patientDiscoveryMap.put("matchFailsTestPatients", "Match Fails for Test Patients");
        
        final Object startDateStr = session.getAttribute("patientDiscoverySummaryQueryStartDate"); 
        final Object endDateStr = session.getAttribute("patientDiscoverySummaryQueryEndDate"); 
        
        final Map<String, List<Object>> headingParameters = new LinkedHashMap<String, List<Object>>();
        
        LinkedHashMap<String,Object> filters = new LinkedHashMap<String,Object>();
        //List all the possible optional filters for Patient Discovery Audit Report.
        filters.put("Start Date",this.getReportHelper().getFormattedDate((Date)startDateStr));
        filters.put("End Date",this.getReportHelper().getFormattedDate((Date)endDateStr));
        filters.put("eHealth Exchange Organization",session.getAttribute("patientDiscoverySummaryQueryOrgName"));
        filters.put("VA Facility", session.getAttribute("facilityName"));
        
        //Iterate over all potential filters and add non-default ones to
        //the heading parameters.
        for (Map.Entry<String,Object> filter : filters.entrySet()){
            String filterName = filter.getKey();
            Object value = filter.getValue();
            List<Object> headingValue = new ArrayList<Object>();
            if (!(value == null) && !("".equals(value)) && !("Unknown".equals(value)) 
                    && !("ALL".equals(value))){
                headingValue.add(value);
                headingParameters.put(filterName, headingValue);
            }
        }
        
//        if ((Boolean)session.getAttribute("testPatients")){
//            List<Object> headingValue = new ArrayList<Object>();
//            headingValue.add("Yes");
//            headingParameters.put("Include test patients", headingValue);
//        }
        
        //Handle the calculation of optional row totals here.
        final Map<String, List<Object>> optionalRows = new LinkedHashMap<String, List<Object>>();
        List<Object> auditCount = new ArrayList<Object>();
            auditCount.add(session.getAttribute("totalCount"));    
        
        optionalRows.put("Total audits", auditCount);    
        
        // Create workbook
        final Workbook wb = this.getExcelExporter().exportToExcel(
        "Patient Discovery Audit Summary Report", "Patient Discovery Audit Summary Report",
        patientDiscoveryMap, results, headingParameters, optionalRows); 
        // Write Excel to Stream 
        this.getExcelExporter().writeExcelToStream(wb, response);

    }
    
    /**
     * Get the excel exporter class from Spring.
     *
     * @return the excel exporter object
     */
    public ExcelExporter getExcelExporter() {
        final ExcelExporter excelExporter = this.getBean("excelExporter",
                ExcelExporter.class);
        return excelExporter;
    }
    
    /**
     * Get the report helper from Spring.
     * @return ReportHelper Report helper
     */
    public ReportHelper getReportHelper() {
        return this.getBean("reportHelper", ReportHelper.class);
    }
   
    /**
     * Get the consent management helper from Spring.
     * @return ConsentManagementHelper Consent management helper
     */
    public ConsentManagementHelper getCmsHelper() {
        return this.getBean("cmsHelper", ConsentManagementHelper.class);
    }

    /**
     * Get the facility helper from Spring.
     * @return FacilityHelper helper
     */
    public FacilityHelper getFacilityHelper() {
        return this.getBean("facilityHelper", FacilityHelper.class);
    }

    @Override
    public void unspecified(final HttpServletRequest request,
            final HttpServletResponse response) throws ServletException,
            IOException {

        final HttpSession session = request.getSession(false);
        final Date sd = (Date) session.getAttribute("patientDiscoverySummaryQueryStartDate");
        Date ed = (Date) session.getAttribute("patientDiscoverySummaryQueryEndDate");
        final String organization = (String) session.getAttribute("patientDiscoverySummaryQueryOrganization");
        final String patientFacility = (String) session.getAttribute("facility");
        final GetAuditsSummary getAuditsSummaryRequest = new GetAuditsSummary();
        
        // Set start date
        if (NullChecker.isNotEmpty(sd)) {
            getAuditsSummaryRequest.setFromDate(GregorianDateUtil.getGregorianCalendarByDate(sd));
        }
        // Set end date
        if (NullChecker.isNotEmpty(ed)) {
            if (ed.equals(sd)) {
                ed = DateUtil.addTime(ed, Calendar.HOUR, 24);
            }
            getAuditsSummaryRequest.setToDate(GregorianDateUtil.getGregorianCalendarByDate(ed));
        }
        // Set patient facility
         if (!NullChecker.isNullOrEmpty(patientFacility)) {
             final StringValuesType patientFacilyNumber = new StringValuesType();
             patientFacilyNumber.setNotIn(false);
             final List<String> patientFacilyNumberList = patientFacilyNumber.getValue();
             patientFacilyNumberList.add(patientFacility);
             getAuditsSummaryRequest.setPatientFacilityNumbers(patientFacilyNumber);
         }
        // Set partner organization
        final StringValuesType orgTypes = new StringValuesType();
        if (NullChecker.isNotEmpty(organization)) {
            orgTypes.getValue().add(organization);
        }
        orgTypes.setNotIn(false);
        getAuditsSummaryRequest.setRemoteOrganizationIds(orgTypes);
        getAuditsSummaryRequest.setOrganizationIds(orgTypes);
        
        final ActionValuesType actionsType = new ActionValuesType();
        actionsType.getValue().add(ActionType.ANNOUNCE);
        actionsType.getValue().add(ActionType.CHECK_POLICY);
        actionsType.getValue().add(ActionType.MPI_FIND_MATCH);
        actionsType.getValue().add(ActionType.ADD_PATIENT_CORRELATION);
        actionsType.setNotIn(false);
        getAuditsSummaryRequest.setActions(actionsType);
     
        // Group by
        final GroupByFieldsType groupByFieldsType = new GroupByFieldsType();
        final List<FieldType> fieldTypeList = groupByFieldsType.getGroupByField();
        fieldTypeList.add(FieldType.REMOTE_ORGANIZATION_ID);
        fieldTypeList.add(FieldType.PATIENT_FACILITY_NUMBER);
        fieldTypeList.add(FieldType.PATIENT_FACILITY_NAME);
        fieldTypeList.add(FieldType.REMOTE_ORGANIZATION_NAME);
        getAuditsSummaryRequest.setGroupByFields(groupByFieldsType);

        try {
            // Get summary list from Exchange
            GetAuditsSummaryResponse getAuditsSummaryResponse = this.adapterAuditManager.getAuditSummary(getAuditsSummaryRequest);

            // Construct results for the jsp page
            final List<Map<String, Object>> results = new ArrayList<Map<String, Object>>();
            long totalCount = 0l;
            final AuditSummariesType auditSummariesType = getAuditsSummaryResponse.getAuditSummaries();

            List<AuditSummaryType> auditSummary = new ArrayList<AuditSummaryType>();
            if (auditSummariesType != null) {
                auditSummary = auditSummariesType.getAuditSummary();
            }
            for (final AuditSummaryType auditSummaryType2 : auditSummary) {
                final AuditSummaryType auditSummaryType = auditSummaryType2;
                final Map<String, Object> resultMap = new HashMap<String, Object>();
                final long count = auditSummaryType.getCount();
                final SummaryFieldsType summaryFields = auditSummaryType.getSummaryFields();
                final List<String> summaryField = summaryFields.getSummaryField();

                if (NullChecker.isEmpty(summaryField)) {
                    continue;
                }

                if (summaryField.size() < 4) {
                    continue;
                }

                // Partner organization
                OrganizationType organizationType = null;
                if (NullChecker.isNotEmpty(summaryField.get(0))) {
                    organizationType = (this.getCmsHelper().getOrganizationByHomeCommunityId(summaryField.get(0).replace("urn:oid:", "")));
                }
                if (!NullChecker.isNullOrEmpty(organizationType)) {
                    resultMap.put("partnerOrg", ReportDataProcessor.nullEmptyReplaceWithUnknown(organizationType.getOrgName()));
                } else {
                    resultMap.put("partnerOrg", ReportDataProcessor.nullEmptyReplaceWithUnknown(summaryField.get(3)));
                }

                // Patient facility
                final String adapterVAFacility = summaryField.get(1);
                String vaFacility;
                if (ReportDataProcessor.isValidStation(adapterVAFacility)) {
                    vaFacility = this.getFacilityHelper().getFacilityNameByStationId(adapterVAFacility);
                    if (NullChecker.isEmpty(vaFacility)) {
                        vaFacility = summaryField.get(2);
                    }
                } else {
                    vaFacility = summaryField.get(2);
                }
                resultMap.put("vaFacility", ReportDataProcessor.fixStation(vaFacility));

                // Audits count
                resultMap.put("auditsCount", count);

                // The rest of the values are simply copied from auditSummary
                resultMap.put("countUniqueRealPatients", summaryField.get(4));
                resultMap.put("countUniqueTestPatients", summaryField.get(5));
                resultMap.put("matchesFoundRealPatients", summaryField.get(6));
                resultMap.put("matchesFoundTestPatients", summaryField.get(7));
                resultMap.put("matchFailsRealPatients", summaryField.get(8));
                resultMap.put("matchFailsTestPatients", summaryField.get(9));

                results.add(resultMap);
                totalCount += count;
            }
            session.setAttribute("totalCount", totalCount);
            session.setAttribute("patientDiscoverySummaryResults", results);
        } catch (final AuditException ex) {
            throw new RuntimeException(ex);
        }
        this.forward(request, response, "show");
    }
}